@@ -1,11 +1,14 @@
 #include "common.h"
+
 #include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
 
 #include "debug.h"
 
 #if IMLIB2_DEBUG
 
-unsigned int        __imlib_debug = 0;
+__EXPORT__ unsigned int __imlib_debug = 0;
 
 static FILE        *opt_fout = NULL;
 
@@ -21,7 +24,8 @@ __attribute__((constructor))
    if (!s)
       return;
 
-   sscanf(s, "%d:%d", &p1, &p2);
+   p1 = p2 = 0;
+   sscanf(s, "%i:%i", &p1, &p2);
 
    __imlib_debug = p1;
    opt_fout = (p2) ? stderr : stdout;
@@ -33,7 +37,7 @@ __attribute__((constructor))
 #include <sys/time.h>
 #endif
 
-unsigned int
+__EXPORT__ unsigned int
 __imlib_time_us(void)
 {
 #if USE_MONOTONIC_CLOCK
@@ -51,7 +55,7 @@ __imlib_time_us(void)
 #endif
 }
 
-void
+__EXPORT__ void
 __imlib_printf(const char *pfx, const char *fmt, ...)
 {
    char                fmtx[1024];
