@@ -1,15 +1,11 @@
-#include "config.h"
+#include "common.h"
 
 #include <ft2build.h>
 #include FT_FREETYPE_H
 #include FT_GLYPH_H
 #include <math.h>
-#include <string.h>
-#include <sys/types.h>
 
 #include "blend.h"
-#include "colormod.h"
-#include "common.h"
 #include "font.h"
 #include "image.h"
 #include "rgbadraw.h"
@@ -68,14 +64,14 @@ __imlib_font_cache_glyph_get(ImlibFont * fn, FT_UInt index)
 
 void
 __imlib_render_str(ImlibImage * im, ImlibFont * fn, int drx, int dry,
-                   const char *text, DATA32 pixel, int dir, double angle,
+                   const char *text, uint32_t pixel, int dir, double angle,
                    int *retw, int *reth, int blur,
                    int *nextx, int *nexty, ImlibOp op, int clx, int cly,
                    int clw, int clh)
 {
    int                 w, h, ascent;
    ImlibImage         *im2;
-   DATA32             *data;
+   uint32_t           *data;
    int                 nx, ny;
 
    __imlib_font_query_advance(fn, text, &w, NULL);
@@ -83,7 +79,7 @@ __imlib_render_str(ImlibImage * im, ImlibFont * fn, int drx, int dry,
 
    if (!IMAGE_DIMENSIONS_OK(w, h))
       return;
-   data = calloc(w * h, sizeof(DATA32));
+   data = calloc(w * h, sizeof(uint32_t));
    if (!data)
       return;
    /* TODO check if this is the right way of rendering. Esp for huge sizes */
@@ -93,7 +89,7 @@ __imlib_render_str(ImlibImage * im, ImlibFont * fn, int drx, int dry,
         free(data);
         return;
      }
-   SET_FLAG(im2->flags, F_HAS_ALPHA);
+   im2->has_alpha = 1;
 
    ascent = __imlib_font_max_ascent_get(fn);
 
@@ -128,7 +124,8 @@ __imlib_render_str(ImlibImage * im, ImlibFont * fn, int drx, int dry,
      }
    if (angle == 0.0)
      {
-        __imlib_BlendImageToImage(im2, im, 0, 1, IMAGE_HAS_ALPHA(im), 0, 0,
+        __imlib_BlendImageToImage(im2, im, 0, 1,
+                                  im->has_alpha, 0, 0,
                                   im2->w, im2->h, drx, dry, im2->w, im2->h,
                                   NULL, op, clx, cly, clw, clh);
      }
@@ -150,7 +147,8 @@ __imlib_render_str(ImlibImage * im, ImlibFont * fn, int drx, int dry,
              xx -= ca * im2->w;
              yy -= ca * im2->h;
           }
-        __imlib_BlendImageToImageSkewed(im2, im, 1, 1, IMAGE_HAS_ALPHA(im), 0,
+        __imlib_BlendImageToImageSkewed(im2, im, 1, 1,
+                                        im->has_alpha, 0,
                                         0, im2->w, im2->h, xx, yy, (w * ca),
                                         (w * sa), 0, 0, NULL, op, clx, cly, clw,
                                         clh);
@@ -245,14 +243,14 @@ __imlib_render_str(ImlibImage * im, ImlibFont * fn, int drx, int dry,
 }
 
 void
-__imlib_font_draw(ImlibImage * dst, DATA32 col, ImlibFont * fn, int x, int y,
+__imlib_font_draw(ImlibImage * dst, uint32_t col, ImlibFont * fn, int x, int y,
                   const char *text, int *nextx, int *nexty, int clx, int cly,
                   int clw, int clh)
 {
    int                 pen_x, pen_y;
    int                 chr;
    int                 ext_x, ext_y, ext_w, ext_h;
-   DATA32             *im;
+   uint32_t           *im;
    int                 im_w, im_h;
    int                 lut[256];
    int                 ii;
@@ -319,7 +317,7 @@ __imlib_font_draw(ImlibImage * dst, DATA32 col, ImlibFont * fn, int x, int y,
 
         if (chr_x < (ext_x + ext_w))
           {
-             DATA8              *data;
+             uint8_t            *data;
              int                 i, j, w, h;
 
              data = fg->glyph_out->bitmap.buffer;
@@ -355,9 +353,9 @@ __imlib_font_draw(ImlibImage * dst, DATA32 col, ImlibFont * fn, int x, int y,
                                    }
                                  if (in_w < w)
                                    {
-                                      DATA8              *src_ptr;
-                                      DATA32             *dst_ptr;
-                                      DATA32             *dst_end_ptr;
+                                      uint8_t            *src_ptr;
+                                      uint32_t           *dst_ptr;
+                                      uint32_t           *dst_end_ptr;
 
                                       src_ptr = data + (i * j) + in_x;
                                       dst_ptr = im + (dy * im_w) + dx;
