@@ -1,14 +1,18 @@
+#include "config.h"
+#include <Imlib2.h>
 #include "common.h"
 
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <sys/mman.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include "Imlib2.h"
 #include "debug.h"
 #include "file.h"
 #include "image.h"
@@ -17,11 +21,23 @@
 #include "x11_pixmap.h"
 #endif
 
+struct _ImlibImageFileInfo {
+   struct _ImlibImageFileInfo *next;
+   char               *name;
+   FILE               *fp;
+   const void         *fdata;
+   off_t               fsize;
+   /* vvv Private vvv */
+   bool                keep_fp;
+   bool                keep_mem;
+   /* ^^^ Private ^^^ */
+};
+
 #define DBG_PFX "IMG"
 #define DP(fmt...) DC(DBG_LOAD, fmt)
 
 /* Imlib loader context */
-struct _imlibldctx {
+struct _ImlibLoaderCtx {
    ImlibProgressFunction progress;
    char                granularity;
    int                 pct, area, row;
@@ -32,7 +48,7 @@ static ImlibImage  *images = NULL;
 
 static int          cache_size = 4096 * 1024;
 
-__EXPORT__ DATA32  *
+__EXPORT__ uint32_t *
 __imlib_AllocateData(ImlibImage * im)
 {
    int                 w = im->w;
@@ -42,9 +58,9 @@ __imlib_AllocateData(ImlibImage * im)
       return NULL;
 
    if (im->data_memory_func)
-      im->data = im->data_memory_func(NULL, w * h * sizeof(DATA32));
+      im->data = im->data_memory_func(NULL, w * h * sizeof(uint32_t));
    else
-      im->data = malloc(w * h * sizeof(DATA32));
+      im->data = malloc(w * h * sizeof(uint32_t));
 
    return im->data;
 }
@@ -52,33 +68,113 @@ __imlib_AllocateData(ImlibImage * im)
 __EXPORT__ void
 __imlib_FreeData(ImlibImage * im)
 {
-   if (im->data)
-     {
-        if (im->data_memory_func)
-           im->data_memory_func(im->data, im->w * im->h * sizeof(DATA32));
-        else
-           free(im->data);
+   if (!im->data)
+      return;
 
-        im->data = NULL;
-     }
-   im->w = 0;
-   im->h = 0;
+   if (im->data_memory_func)
+      im->data_memory_func(im->data, im->w * im->h * sizeof(uint32_t));
+   else
+      free(im->data);
+
+   im->data = NULL;
 }
 
 __EXPORT__ void
 __imlib_ReplaceData(ImlibImage * im, unsigned int *new_data)
 {
-   if (im->data)
-     {
-        if (im->data_memory_func)
-           im->data_memory_func(im->data, im->w * im->h * sizeof(DATA32));
-        else
-           free(im->data);
-     }
+   __imlib_FreeData(im);
    im->data = new_data;
    im->data_memory_func = NULL;
 }
 
+static int
+__imlib_FileContextOpen(ImlibImageFileInfo * fi, FILE * fp,
+                        const void *fdata, off_t fsize)
+{
+   struct stat         st;
+
+   if (fp)
+     {
+        fi->keep_fp = true;
+        fi->fp = fp;
+     }
+   else if (fdata)
+     {
+        fi->keep_mem = true;
+     }
+   else
+     {
+        fi->fp = fopen(fi->name, "rb");
+        if (!fi->fp)
+           return -1;
+     }
+
+   if (fi->fsize == 0)
+     {
+        if (fsize == 0)
+          {
+             __imlib_FileStat(fi->name, &st);
+             fsize = st.st_size;
+          }
+        fi->fsize = fsize;
+     }
+
+   if (!fdata)
+     {
+        fdata = mmap(NULL, fi->fsize, PROT_READ, MAP_SHARED, fileno(fi->fp), 0);
+        if (fdata == MAP_FAILED)
+           return -1;
+     }
+
+   fi->fdata = fdata;
+
+   return 0;
+}
+
+static void
+__imlib_FileContextClose(ImlibImageFileInfo * fi)
+{
+   if (fi->fdata && !fi->keep_mem)
+     {
+        munmap((void *)fi->fdata, fi->fsize);
+        fi->fdata = NULL;
+     }
+
+   if (fi->fp && !fi->keep_fp)
+     {
+        fclose(fi->fp);
+        fi->fp = NULL;
+     }
+}
+
+static int
+__imlib_ImageFileContextPush(ImlibImage * im, char *file)
+{
+   ImlibImageFileInfo *fi;
+
+   fi = calloc(1, sizeof(ImlibImageFileInfo));
+   if (!fi)
+      return -1;
+
+   fi->name = file;             /* file must have been alloced, will be freed */
+
+   fi->next = im->fi;
+   im->fi = fi;
+
+   return 0;
+}
+
+static void
+__imlib_ImageFileContextPop(ImlibImage * im)
+{
+   ImlibImageFileInfo *fi = im->fi;
+
+   im->fi = fi->next;
+
+   free(fi->name);
+   free(fi);
+}
+
 /* create an image data struct and fill it in */
 static ImlibImage  *
 __imlib_ProduceImage(void)
@@ -86,7 +182,7 @@ __imlib_ProduceImage(void)
    ImlibImage         *im;
 
    im = calloc(1, sizeof(ImlibImage));
-   im->flags = F_FORMAT_IRRELEVANT | F_BORDER_IRRELEVANT | F_ALPHA_IRRELEVANT;
+   im->flags = F_FORMAT_IRRELEVANT;
 
    return im;
 }
@@ -101,14 +197,18 @@ __imlib_ConsumeImage(ImlibImage * im)
 
    __imlib_FreeAllTags(im);
 
-   if (im->real_file && im->real_file != im->file)
-      free(im->real_file);
-   free(im->file);
+   if (im->file && (!im->fi || im->fi->name != im->file))
+      free(im->file);
    free(im->key);
-   if ((IMAGE_FREE_DATA(im)) && (im->data))
+   if (im->data && !IM_FLAG_ISSET(im, F_DONT_FREE_DATA))
       __imlib_FreeData(im);
    free(im->format);
 
+   if (im->fi)
+      __imlib_ImageFileContextPop(im);
+
+   free(im->pframe);
+
    free(im);
 }
 
@@ -122,19 +222,20 @@ __imlib_FindCachedImage(const char *file, int frame)
    for (im = images, im_prev = NULL; im; im_prev = im, im = im->next)
      {
         /* if the filenames match and it's valid */
-        if (!strcmp(file, im->file) && IMAGE_IS_VALID(im) &&
-            frame == im->frame_num)
+        if (strcmp(file, im->file) || IM_FLAG_ISSET(im, F_INVALID))
+           continue;
+        if (frame != im->frame)
+           continue;
+
+        /* move the image to the head of the image list */
+        if (im_prev)
           {
-             /* move the image to the head of the image list */
-             if (im_prev)
-               {
-                  im_prev->next = im->next;
-                  im->next = images;
-                  images = im;
-               }
-             DP(" got %p: '%s' frame %d\n", im, im->real_file, im->frame_num);
-             return im;
+             im_prev->next = im->next;
+             im->next = images;
+             images = im;
           }
+        DP(" got %p: '%s' frame %d\n", im, im->fi->name, im->frame);
+        return im;
      }
    DP(" got none\n");
    return NULL;
@@ -144,7 +245,7 @@ __imlib_FindCachedImage(const char *file, int frame)
 static void
 __imlib_AddImageToCache(ImlibImage * im)
 {
-   DP("%s: %p: '%s' frame %d\n", __func__, im, im->real_file, im->frame_num);
+   DP("%s: %p: '%s' frame %d\n", __func__, im, im->fi->name, im->frame);
    im->next = images;
    images = im;
 }
@@ -156,7 +257,7 @@ __imlib_RemoveImageFromCache(ImlibImage * im_del)
    ImlibImage         *im, *im_prev;
 
    im = im_del;
-   DP("%s: %p: '%s' frame %d\n", __func__, im, im->real_file, im->frame_num);
+   DP("%s: %p: '%s' frame %d\n", __func__, im, im->fi->name, im->frame);
 
    for (im = images, im_prev = NULL; im; im_prev = im, im = im->next)
      {
@@ -186,14 +287,14 @@ __imlib_CurrentCacheSize(void)
         /* mayaswell clean out stuff thats invalid that we dont need anymore */
         if (im->references == 0)
           {
-             if (!(IMAGE_IS_VALID(im)))
+             if (IM_FLAG_ISSET(im, F_INVALID))
                {
                   __imlib_RemoveImageFromCache(im);
                   __imlib_ConsumeImage(im);
                }
              /* it's valid but has 0 ref's - append to cache size count */
              else
-                current_cache += im->w * im->h * sizeof(DATA32);
+                current_cache += im->w * im->h * sizeof(uint32_t);
           }
      }
 
@@ -217,7 +318,7 @@ __imlib_CleanupImageCache(void)
    for (im = images; im; im = im_next)
      {
         im_next = im->next;
-        if ((im->references <= 0) && (!(IMAGE_IS_VALID(im))))
+        if (im->references <= 0 && IM_FLAG_ISSET(im, F_INVALID))
           {
              __imlib_RemoveImageFromCache(im);
              __imlib_ConsumeImage(im);
@@ -261,47 +362,10 @@ __imlib_GetCacheSize(void)
    return cache_size;
 }
 
-static int
-__imlib_ErrorFromErrno(int err, int save)
-{
-   switch (err)
-     {
-     default:
-        return IMLIB_LOAD_ERROR_UNKNOWN;
-        /* standrad fopen() type errors translated */
-     case 0:
-        return IMLIB_LOAD_ERROR_NONE;
-     case EEXIST:
-        return IMLIB_LOAD_ERROR_FILE_DOES_NOT_EXIST;
-     case EISDIR:
-        return IMLIB_LOAD_ERROR_FILE_IS_DIRECTORY;
-     case EACCES:
-     case EROFS:
-        return (save) ? IMLIB_LOAD_ERROR_PERMISSION_DENIED_TO_WRITE :
-           IMLIB_LOAD_ERROR_PERMISSION_DENIED_TO_READ;
-     case ENAMETOOLONG:
-        return IMLIB_LOAD_ERROR_PATH_TOO_LONG;
-     case ENOENT:
-        return IMLIB_LOAD_ERROR_PATH_COMPONENT_NON_EXISTANT;
-     case ENOTDIR:
-        return IMLIB_LOAD_ERROR_PATH_COMPONENT_NOT_DIRECTORY;
-     case EFAULT:
-        return IMLIB_LOAD_ERROR_PATH_POINTS_OUTSIDE_ADDRESS_SPACE;
-     case ELOOP:
-        return IMLIB_LOAD_ERROR_TOO_MANY_SYMBOLIC_LINKS;
-     case ENOMEM:
-        return IMLIB_LOAD_ERROR_OUT_OF_MEMORY;
-     case EMFILE:
-        return IMLIB_LOAD_ERROR_OUT_OF_FILE_DESCRIPTORS;
-     case ENOSPC:
-        return IMLIB_LOAD_ERROR_OUT_OF_DISK_SPACE;
-     }
-}
-
 /* create a new image struct from data passed that is wize w x h then return */
 /* a pointer to that image sturct */
 ImlibImage         *
-__imlib_CreateImage(int w, int h, DATA32 * data)
+__imlib_CreateImage(int w, int h, uint32_t * data)
 {
    ImlibImage         *im;
 
@@ -310,7 +374,7 @@ __imlib_CreateImage(int w, int h, DATA32 * data)
    im->h = h;
    im->data = data;
    im->references = 1;
-   SET_FLAG(im->flags, F_UNCACHEABLE);
+   IM_FLAG_SET(im, F_UNCACHEABLE);
    return im;
 }
 
@@ -319,34 +383,19 @@ __imlib_LoadImageWrapper(const ImlibLoader * l, ImlibImage * im, int load_data)
 {
    int                 rc;
 
-   DP("%s: fmt='%s' file='%s'(%s), imm=%d\n", __func__,
-      l->formats[0], im->file, im->real_file, load_data);
+   DP("%s: fmt='%s' file='%s'(%s) frame=%d, imm=%d\n", __func__,
+      l->name, im->file, im->fi->name, im->frame, load_data);
 
 #if IMLIB2_DEBUG
    unsigned int        t0 = __imlib_time_us();
 #endif
 
-   if (l->load2)
+   if (l->module->load)
      {
-        FILE               *fp = NULL;
+        if (!im->format)
+           im->format = strdup(l->name);
 
-        if (!im->fp)
-          {
-             fp = im->fp = fopen(im->real_file, "rb");
-             if (!im->fp)
-                return 0;
-          }
-        rc = l->load2(im, load_data);
-
-        if (fp)
-           fclose(fp);
-     }
-   else if (l->load)
-     {
-        if (im->lc)
-           rc = l->load(im, im->lc->progress, im->lc->granularity, 1);
-        else
-           rc = l->load(im, NULL, 0, load_data);
+        rc = l->module->load(im, load_data);
      }
    else
      {
@@ -354,42 +403,24 @@ __imlib_LoadImageWrapper(const ImlibLoader * l, ImlibImage * im, int load_data)
      }
 
    DP("%s: %-4s: %s: Elapsed time: %.3f ms\n", __func__,
-      l->formats[0], im->file, 1e-3 * (__imlib_time_us() - t0));
+      l->name, im->fi->name, 1e-3 * (__imlib_time_us() - t0));
 
    if (rc <= LOAD_FAIL)
      {
         /* Failed - clean up */
         DP("%s: Failed (rc=%d)\n", __func__, rc);
 
-        if (im->w != 0 || im->h != 0)
-          {
-             DP("%s: Loader %s: Didn't clear w/h=%d/%d\n", __func__,
-                l->formats[0], im->w, im->h);
-             im->w = im->h = 0;
-          }
-        if (im->data)
-          {
-             DP("%s: Loader %s: Free im->data\n", __func__, l->formats[0]);
-             __imlib_FreeData(im);
-          }
-        if (im->format)
-          {
-             DP("%s: Loader %s: Free im->format\n", __func__, l->formats[0]);
-             free(im->format);
-             im->format = NULL;
-          }
-     }
-   else
-     {
-        if (!im->format && l->formats && l->formats[0])
-           im->format = strdup(l->formats[0]);
+        im->w = im->h = 0;
+        __imlib_FreeData(im);
+        free(im->format);
+        im->format = NULL;
      }
 
    return rc;
 }
 
 static void
-__imlib_LoadCtxInit(ImlibImage * im, ImlibLdCtx * lc,
+__imlib_LoadCtxInit(ImlibImage * im, ImlibLoaderCtx * lc,
                     ImlibProgressFunction prog, int gran)
 {
    im->lc = lc;
@@ -401,89 +432,88 @@ __imlib_LoadCtxInit(ImlibImage * im, ImlibLdCtx * lc,
    lc->n_pass = 1;
 }
 
-__EXPORT__ int
-__imlib_LoadEmbedded(ImlibLoader * l, ImlibImage * im, const char *file,
-                     int load_data)
+static int
+__imlib_LoadErrorToErrno(int loader_ret, int save)
 {
-   int                 rc;
-   struct stat         st;
-   char               *file_save;
-   FILE               *fp_save;
-   off_t               fsize_save;
-
-   if (!l || !im)
-      return 0;
-
-   /* remember the original filename */
-   file_save = im->real_file;
-   im->real_file = strdup(file);
-   fp_save = im->fp;
-   im->fp = NULL;
-   fsize_save = im->fsize;
-   __imlib_FileStat(file, &st);
-   im->fsize = st.st_size;
-
-   rc = __imlib_LoadImageWrapper(l, im, load_data);
-
-   im->fp = fp_save;
-   free(im->real_file);
-   im->real_file = file_save;
-   im->fsize = fsize_save;
-
-   return rc;
+   switch (loader_ret)
+     {
+     default:                  /* We should not go here */
+        return IMLIB_ERR_INTERNAL;
+     case LOAD_SUCCESS:
+        return 0;
+     case LOAD_FAIL:
+        return (save) ? IMLIB_ERR_NO_SAVER : IMLIB_ERR_NO_LOADER;
+     case LOAD_OOM:
+        return ENOMEM;
+     case LOAD_BADFILE:
+        return errno;
+     case LOAD_BADIMAGE:
+        return IMLIB_ERR_BAD_IMAGE;
+     case LOAD_BADFRAME:
+        return IMLIB_ERR_BAD_FRAME;
+     }
 }
 
 ImlibImage         *
 __imlib_LoadImage(const char *file, ImlibLoadArgs * ila)
 {
    ImlibImage         *im;
-   ImlibLoader        *best_loader;
+   ImlibLoader       **loaders, *best_loader, *l, *previous_l;
    int                 err, loader_ret;
-   ImlibLdCtx          ilc;
+   ImlibLoaderCtx      ilc;
    struct stat         st;
    char               *im_file, *im_key;
 
    if (!file || file[0] == '\0')
       return NULL;
 
-   /* see if we already have the image cached */
-   im = __imlib_FindCachedImage(file, ila->frame);
-
-   /* if we found a cached image and we should always check that it is */
-   /* accurate to the disk conents if they changed since we last loaded */
-   /* and that it is still a valid image */
-   if ((im) && (IMAGE_IS_VALID(im)))
+   if (!ila->nocache)
      {
-        if (IMAGE_ALWAYS_CHECK_DISK(im))
-          {
-             time_t              current_modified_time;
+        /* see if we already have the image cached */
+        im = __imlib_FindCachedImage(file, ila->frame);
 
-             current_modified_time = ila->fp ?
-                __imlib_FileModDateFd(fileno(ila->fp)) :
-                __imlib_FileModDate(im->real_file);
-             /* if the file on disk is newer than the cached one */
-             if (current_modified_time != im->moddate)
+        /* if we found a cached image and we should always check that it is */
+        /* accurate to the disk conents if they changed since we last loaded */
+        /* and that it is still a valid image */
+        if (im && !IM_FLAG_ISSET(im, F_INVALID))
+          {
+             if (IM_FLAG_ISSET(im, F_ALWAYS_CHECK_DISK))
                {
-                  /* invalidate image */
-                  SET_FLAG(im->flags, F_INVALID);
+                  time_t              current_modified_time;
+
+                  current_modified_time = ila->fp ?
+                     __imlib_FileModDateFd(fileno(ila->fp)) :
+                     __imlib_FileModDate(im->fi->name);
+                  /* if the file on disk is newer than the cached one */
+                  if (current_modified_time != im->moddate)
+                    {
+                       /* invalidate image */
+                       IM_FLAG_SET(im, F_INVALID);
+                    }
+                  else
+                    {
+                       /* image is ok to re-use - program is just being stupid loading */
+                       /* the same data twice */
+                       im->references++;
+                       return im;
+                    }
                }
              else
                {
-                  /* image is ok to re-use - program is just being stupid loading */
-                  /* the same data twice */
                   im->references++;
                   return im;
                }
           }
-        else
-          {
-             im->references++;
-             return im;
-          }
      }
 
    im_file = im_key = NULL;
-   if (ila->fp)
+   if (ila->fdata)
+     {
+        err = 0;
+        st.st_size = ila->fsize;
+        st.st_mtime = st.st_ctime = 0;
+     }
+   else if (ila->fp)
      {
         err = fstat(fileno(ila->fp), &st);
      }
@@ -502,11 +532,11 @@ __imlib_LoadImage(const char *file, ImlibLoadArgs * ila)
      }
 
    if (err)
-      err = IMLIB_LOAD_ERROR_FILE_DOES_NOT_EXIST;
-   else if (__imlib_StatIsDir(&st))
-      err = IMLIB_LOAD_ERROR_FILE_IS_DIRECTORY;
+      err = errno;
    else if (st.st_size == 0)
-      err = IMLIB_LOAD_ERROR_UNKNOWN;
+      err = IMLIB_ERR_BAD_IMAGE;
+   else if (!ila->fdata && __imlib_StatIsDir(&st))
+      err = EISDIR;
 
    if (err)
      {
@@ -520,19 +550,13 @@ __imlib_LoadImage(const char *file, ImlibLoadArgs * ila)
    /* so produce a new one and load an image into that */
    im = __imlib_ProduceImage();
    im->file = strdup(file);
-   im->fsize = st.st_size;
-   im->real_file = im_file ? im_file : im->file;
    im->key = im_key;
-   im->frame_num = ila->frame;
+   im->frame = ila->frame;
 
-   if (ila->fp)
-      im->fp = ila->fp;
-   else
-      im->fp = fopen(im->real_file, "rb");
-
-   if (!im->fp)
+   if (__imlib_ImageFileContextPush(im, im_file ? im_file : im->file) ||
+       __imlib_FileContextOpen(im->fi, ila->fp, ila->fdata, st.st_size))
      {
-        ila->err = __imlib_ErrorFromErrno(errno, 0);
+        ila->err = errno;
         __imlib_ConsumeImage(im);
         return NULL;
      }
@@ -547,96 +571,87 @@ __imlib_LoadImage(const char *file, ImlibLoadArgs * ila)
         ila->immed = 1;
      }
 
-   loader_ret = LOAD_FAIL;
-
    /* take a guess by extension on the best loader to use */
-   best_loader = __imlib_FindBestLoaderForFile(im->real_file, 0);
-   errno = 0;
-   if (best_loader)
-      loader_ret = __imlib_LoadImageWrapper(best_loader, im, ila->immed);
+   best_loader = __imlib_FindBestLoader(im->fi->name, NULL, 0);
 
-   switch (loader_ret)
+   loader_ret = LOAD_FAIL;
+   loaders = NULL;
+
+   for (l = previous_l = NULL;;)
      {
-        ImlibLoader       **loaders, *l, *previous_l;
-
-     case LOAD_BREAK:          /* Break signaled by progress callback */
-     case LOAD_SUCCESS:        /* Image loaded successfully           */
-        /* Loader accepted image */
-        im->loader = best_loader;
-        break;
-
-     case LOAD_FAIL:           /* Image was not recognized by loader  */
-        loaders = __imlib_GetLoaderList();
-        errno = 0;
-        /* run through all loaders and try load until one succeeds */
-        for (l = *loaders, previous_l = NULL; l; previous_l = l, l = l->next)
+        if (l == NULL && best_loader)
           {
-             /* if its not the best loader that already failed - try load */
-             if (l == best_loader)
-                continue;
-             fflush(im->fp);
-             rewind(im->fp);
-             loader_ret = __imlib_LoadImageWrapper(l, im, ila->immed);
-             if (loader_ret > LOAD_FAIL)
-                break;
+             l = best_loader;   /* First try best_loader, if any */
           }
-
-        /* if we have a loader then its the loader that succeeded */
-        /* move the successful loader to the head of the list */
-        /* as long as it's not already at the head of the list */
-        if (l)
+        else if (!loaders)
           {
+             loaders = __imlib_GetLoaderList();
+             l = *loaders;
+             if (best_loader && l == best_loader)
+                continue;       /* Skip best_loader that already failed */
+          }
+        else
+          {
+             previous_l = l;
+             l = l->next;
+             if (best_loader && l == best_loader)
+                continue;       /* Skip best_loader that already failed */
+          }
+        if (!l)
+           break;
+
+        errno = 0;
+        loader_ret = __imlib_LoadImageWrapper(l, im, ila->immed);
+
+        switch (loader_ret)
+          {
+          case LOAD_BREAK:     /* Break signaled by progress callback */
+          case LOAD_SUCCESS:   /* Image loaded successfully           */
+             /* Loader accepted image - done */
              im->loader = l;
+
+             /* move the successful loader to the head of the list */
              if (previous_l)
                {
                   previous_l->next = l->next;
                   l->next = *loaders;
                   *loaders = l;
                }
-          }
-        break;
+             break;
 
-     default:                  /* We should not go here */
-     case LOAD_OOM:            /* Could not allocate memory           */
-     case LOAD_BADFILE:        /* File could not be accessed          */
-        /* Unlikely that another loader will succeed */
-     case LOAD_BADIMAGE:       /* Image is corrupt                    */
-     case LOAD_BADFRAME:       /* Requested frame not found           */
-        /* Signature was good but file was somehow not */
+          case LOAD_FAIL:
+             /* Image was not recognized by loader - try next */
+             if (im->fi->fp)
+               {
+                  fflush(im->fi->fp);
+                  rewind(im->fi->fp);
+               }
+             continue;
+
+          default:             /* We should not go here */
+          case LOAD_OOM:       /* Could not allocate memory           */
+          case LOAD_BADFILE:   /* File could not be accessed          */
+             /* Unlikely that another loader will succeed */
+          case LOAD_BADIMAGE:  /* Image is invalid                    */
+          case LOAD_BADFRAME:  /* Requested frame not found           */
+             /* Signature was good but file was somehow not */
+             break;
+          }
+
+        /* Done or don't look for other */
         break;
      }
 
    im->lc = NULL;
 
    if (!ila->fp)
-      fclose(im->fp);
-   im->fp = NULL;
+      __imlib_FileContextClose(im->fi);
 
    if (loader_ret <= LOAD_FAIL)
      {
         /* Image loading failed.
          * Free the skeleton image struct we had and return NULL */
-        switch (loader_ret)
-          {
-          default:             /* We should not go here */
-             ila->err = IMLIB_LOAD_ERROR_UNKNOWN;
-             break;
-          case LOAD_FAIL:
-             ila->err = IMLIB_LOAD_ERROR_NO_LOADER_FOR_FILE_FORMAT;
-             break;
-          case LOAD_OOM:
-             ila->err = IMLIB_LOAD_ERROR_OUT_OF_MEMORY;
-             break;
-          case LOAD_BADFILE:
-             ila->err = IMLIB_LOAD_ERROR_PERMISSION_DENIED_TO_READ;
-             break;
-          case LOAD_BADIMAGE:
-             ila->err = IMLIB_LOAD_ERROR_IMAGE_READ;
-             break;
-          case LOAD_BADFRAME:
-             ila->err = IMLIB_LOAD_ERROR_IMAGE_FRAME;
-             break;
-          }
+        ila->err = __imlib_LoadErrorToErrno(loader_ret, 0);
         __imlib_ConsumeImage(im);
         return NULL;
      }
@@ -649,7 +664,7 @@ __imlib_LoadImage(const char *file, ImlibLoadArgs * ila)
    if (!ila->nocache)
       __imlib_AddImageToCache(im);
    else
-      SET_FLAG(im->flags, F_UNCACHEABLE);
+      IM_FLAG_SET(im, F_UNCACHEABLE);
 
    return im;
 }
@@ -657,10 +672,67 @@ __imlib_LoadImage(const char *file, ImlibLoadArgs * ila)
 int
 __imlib_LoadImageData(ImlibImage * im)
 {
-   if (!im->data && im->loader)
-      if (__imlib_LoadImageWrapper(im->loader, im, 1) <= LOAD_FAIL)
-         return 1;              /* Load failed */
-   return im->data == NULL;
+   int                 err;
+
+   if (im->data)
+      return 0;                 /* Ok */
+
+   /* Just checking - it should be impossible that loader is not set */
+   if (!im->loader)
+      return IMLIB_ERR_INTERNAL;
+
+   err = __imlib_FileContextOpen(im->fi, NULL, NULL, 0);
+   if (err)
+      return err;
+   err = __imlib_LoadImageWrapper(im->loader, im, 1);
+
+   __imlib_FileContextClose(im->fi);
+
+   return __imlib_LoadErrorToErrno(err, 0);
+}
+
+__EXPORT__ int
+__imlib_LoadEmbedded(ImlibLoader * l, ImlibImage * im, int load_data,
+                     const char *file)
+{
+   int                 rc;
+
+   if (!l || !im)
+      return LOAD_FAIL;
+
+   __imlib_ImageFileContextPush(im, strdup(file));
+   rc = __imlib_FileContextOpen(im->fi, NULL, NULL, 0);
+   if (rc)
+      return LOAD_FAIL;
+
+   rc = __imlib_LoadImageWrapper(l, im, load_data);
+
+   __imlib_FileContextClose(im->fi);
+   __imlib_ImageFileContextPop(im);
+
+   return rc;
+}
+
+__EXPORT__ int
+__imlib_LoadEmbeddedMem(ImlibLoader * l, ImlibImage * im, int load_data,
+                        const void *fdata, unsigned int fsize)
+{
+   int                 rc;
+
+   if (!l || !im)
+      return LOAD_FAIL;
+
+   __imlib_ImageFileContextPush(im, NULL);
+   rc = __imlib_FileContextOpen(im->fi, NULL, fdata, fsize);
+   if (rc)
+      return LOAD_FAIL;
+
+   rc = __imlib_LoadImageWrapper(l, im, load_data);
+
+   __imlib_FileContextClose(im->fi);
+   __imlib_ImageFileContextPop(im);
+
+   return rc;
 }
 
 __EXPORT__ void
@@ -675,7 +747,7 @@ __imlib_LoadProgressSetPass(ImlibImage * im, int pass, int n_pass)
 __EXPORT__ int
 __imlib_LoadProgress(ImlibImage * im, int x, int y, int w, int h)
 {
-   ImlibLdCtx         *lc = im->lc;
+   ImlibLoaderCtx     *lc = im->lc;
    int                 rc;
 
    lc->area += w * h;
@@ -689,7 +761,8 @@ __imlib_LoadProgress(ImlibImage * im, int x, int y, int w, int h)
 __EXPORT__ int
 __imlib_LoadProgressRows(ImlibImage * im, int row, int nrows)
 {
-   ImlibLdCtx         *lc = im->lc;
+   ImlibLoaderCtx     *lc = im->lc;
+   int                 col = 0;
    int                 rc = 0;
    int                 pct, nrtot;
 
@@ -710,7 +783,7 @@ __imlib_LoadProgressRows(ImlibImage * im, int row, int nrows)
    pct = (100 * nrtot * (lc->pass + 1)) / (im->h * lc->n_pass);
    if (pct == 100 || pct >= lc->pct + lc->granularity)
      {
-        rc = !lc->progress(im, pct, 0, row, im->w, nrows);
+        rc = !lc->progress(im, pct, col, row, im->w, nrows);
         lc->row = nrtot;
         lc->pct += lc->granularity;
      }
@@ -718,6 +791,14 @@ __imlib_LoadProgressRows(ImlibImage * im, int row, int nrows)
    return rc;
 }
 
+__EXPORT__ ImlibImageFrame *
+__imlib_GetFrame(ImlibImage * im)
+{
+   if (!im->pframe)
+      im->pframe = calloc(1, sizeof(ImlibImageFrame));
+   return im->pframe;
+}
+
 /* free and image - if its uncachable and refcoutn is 0 - free it in reality */
 void
 __imlib_FreeImage(ImlibImage * im)
@@ -728,7 +809,7 @@ __imlib_FreeImage(ImlibImage * im)
         /* reduce a reference from the count */
         im->references--;
         /* if its uncachchable ... */
-        if (IMAGE_IS_UNCACHEABLE(im))
+        if (IM_FLAG_ISSET(im, F_UNCACHEABLE))
           {
              /* and we're down to no references for the image then free it */
              if (im->references == 0)
@@ -744,55 +825,52 @@ __imlib_FreeImage(ImlibImage * im)
 void
 __imlib_DirtyImage(ImlibImage * im)
 {
-   SET_FLAG(im->flags, F_INVALID);
+   IM_FLAG_SET(im, F_INVALID);
 #ifdef BUILD_X11
    /* and dirty all pixmaps generated from it */
    __imlib_DirtyPixmapsForImage(im);
 #endif
 }
 
+__EXPORT__ const char *
+__imlib_GetKey(const ImlibImage * im)
+{
+   return im->key;
+}
+
 void
-__imlib_SaveImage(ImlibImage * im, const char *file,
-                  ImlibProgressFunction progress, char progress_granularity,
-                  int *er)
+__imlib_SaveImage(ImlibImage * im, const char *file, ImlibLoadArgs * ila)
 {
    ImlibLoader        *l;
-   char                e, *pfile;
-   ImlibLdCtx          ilc;
+   ImlibLoaderCtx      ilc;
+   int                 loader_ret;
 
    if (!file)
      {
-        if (er)
-           *er = IMLIB_LOAD_ERROR_FILE_DOES_NOT_EXIST;
+        ila->err = ENOENT;
         return;
      }
 
    /* find the laoder for the format - if its null use the extension */
-   l = __imlib_FindBestLoaderForFileFormat(file, im->format, 1);
+   l = __imlib_FindBestLoader(file, im->format, 1);
    /* no loader - abort */
    if (!l)
      {
-        if (er)
-           *er = IMLIB_LOAD_ERROR_NO_LOADER_FOR_FILE_FORMAT;
+        ila->err = IMLIB_ERR_NO_SAVER;
         return;
      }
 
-   if (progress)
-      __imlib_LoadCtxInit(im, &ilc, progress, progress_granularity);
+   if (ila->pfunc)
+      __imlib_LoadCtxInit(im, &ilc, ila->pfunc, ila->pgran);
 
-   /* set the filename to the user supplied one */
-   pfile = im->real_file;
-   im->real_file = strdup(file);
+   __imlib_ImageFileContextPush(im, strdup(file));
 
    /* call the saver */
-   e = l->save(im, progress, progress_granularity);
+   loader_ret = l->module->save(im);
 
-   /* set the filename back to the laoder image filename */
-   free(im->real_file);
-   im->real_file = pfile;
+   __imlib_ImageFileContextPop(im);
 
    im->lc = NULL;
 
-   if (er)
-      *er = __imlib_ErrorFromErrno(e > 0 ? 0 : errno, 1);
+   ila->err = __imlib_LoadErrorToErrno(loader_ret, 1);
 }
