@@ -1,6 +1,7 @@
 #include "common.h"
 
 #include <dlfcn.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -21,28 +22,39 @@ typedef struct {
    const char         *const *ext;
 } KnownLoader;
 
+static const char  *const ext_ani[] = { "ani", NULL };
 static const char  *const ext_argb[] = { "argb", NULL };
 static const char  *const ext_bmp[] = { "bmp", NULL };
-#ifdef BUILD_BZ2_LOADER
-static const char  *const ext_bz2[] = { "bz2", NULL };
-#endif
 static const char  *const ext_ff[] = { "ff", NULL };
 #ifdef BUILD_GIF_LOADER
 static const char  *const ext_gif[] = { "gif", NULL };
 #endif
-static const char  *const ext_ico[] = { "ico", NULL };
-#ifdef BUILD_ID3_LOADER
-static const char  *const ext_id3[] = { "mp3", NULL };
+#ifdef BUILD_HEIF_LOADER
+static const char  *const ext_heif[] =
+   { "heif", "heifs", "heic", "heics", "avci", "avcs", "avif", "avifs", NULL };
 #endif
+static const char  *const ext_ico[] = { "ico", NULL };
 #ifdef BUILD_JPEG_LOADER
 static const char  *const ext_jpeg[] = { "jpg", "jpeg", "jfif", "jfi", NULL };
 #endif
+#ifdef BUILD_J2K_LOADER
+static const char  *const ext_j2k[] = { "jp2", "j2k", NULL };
+#endif
+#ifdef BUILD_JXL_LOADER
+static const char  *const ext_jxl[] = { "jxl", NULL };
+#endif
 static const char  *const ext_lbm[] = { "iff", "ilbm", "lbm", NULL };
 #ifdef BUILD_PNG_LOADER
 static const char  *const ext_png[] = { "png", NULL };
 #endif
 static const char  *const ext_pnm[] =
    { "pnm", "ppm", "pgm", "pbm", "pam", NULL };
+#ifdef BUILD_PS_LOADER
+static const char  *const ext_ps[] = { "ps", "eps", NULL };
+#endif
+#ifdef BUILD_SVG_LOADER
+static const char  *const ext_svg[] = { "svg", NULL };
+#endif
 static const char  *const ext_tga[] = { "tga", NULL };
 #ifdef BUILD_TIFF_LOADER
 static const char  *const ext_tiff[] = { "tiff", "tif", NULL };
@@ -52,31 +64,53 @@ static const char  *const ext_webp[] = { "webp", NULL };
 #endif
 static const char  *const ext_xbm[] = { "xbm", NULL };
 static const char  *const ext_xpm[] = { "xpm", NULL };
+
+#ifdef BUILD_BZ2_LOADER
+static const char  *const ext_bz2[] = { "bz2", NULL };
+#endif
+#ifdef BUILD_LZMA_LOADER
+static const char  *const ext_lzma[] = { "xz", "lzma", NULL };
+#endif
 #ifdef BUILD_ZLIB_LOADER
 static const char  *const ext_zlib[] = { "gz", NULL };
 #endif
+
+#ifdef BUILD_ID3_LOADER
+static const char  *const ext_id3[] = { "mp3", NULL };
+#endif
+
 static const KnownLoader loaders_known[] = {
+   {"ani", ext_ani},
    {"argb", ext_argb},
    {"bmp", ext_bmp},
-#ifdef BUILD_BZ2_LOADER
-   {"bz2", ext_bz2},
-#endif
    {"ff", ext_ff},
 #ifdef BUILD_GIF_LOADER
    {"gif", ext_gif},
 #endif
-   {"ico", ext_ico},
-#ifdef BUILD_ID3_LOADER
-   {"id3", ext_id3},
+#ifdef BUILD_HEIF_LOADER
+   {"heif", ext_heif},
 #endif
+   {"ico", ext_ico},
 #ifdef BUILD_JPEG_LOADER
    {"jpeg", ext_jpeg},
+#endif
+#ifdef BUILD_J2K_LOADER
+   {"j2k", ext_j2k},
+#endif
+#ifdef BUILD_JXL_LOADER
+   {"jxl", ext_jxl},
 #endif
    {"lbm", ext_lbm},
 #ifdef BUILD_PNG_LOADER
    {"png", ext_png},
+#endif
+#ifdef BUILD_PS_LOADER
+   {"ps", ext_ps},
 #endif
    {"pnm", ext_pnm},
+#ifdef BUILD_SVG_LOADER
+   {"svg", ext_svg},
+#endif
    {"tga", ext_tga},
 #ifdef BUILD_TIFF_LOADER
    {"tiff", ext_tiff},
@@ -86,9 +120,18 @@ static const KnownLoader loaders_known[] = {
 #endif
    {"xbm", ext_xbm},
    {"xpm", ext_xpm},
+#ifdef BUILD_BZ2_LOADER
+   {"bz2", ext_bz2},
+#endif
+#ifdef BUILD_LZMA_LOADER
+   {"lzma", ext_lzma},
+#endif
 #ifdef BUILD_ZLIB_LOADER
    {"zlib", ext_zlib},
 #endif
+#ifdef BUILD_ID3_LOADER
+   {"id3", ext_id3},
+#endif
 };
 
 static int
@@ -111,40 +154,39 @@ static ImlibLoader *
 __imlib_ProduceLoader(const char *file)
 {
    ImlibLoader        *l;
-   void                (*l_formats)(ImlibLoader * l);
+   ImlibLoaderModule  *m;
 
    DP("%s: %s\n", __func__, file);
 
    l = malloc(sizeof(ImlibLoader));
-   l->num_formats = 0;
-   l->formats = NULL;
+
    l->handle = dlopen(file, RTLD_NOW | RTLD_LOCAL);
    if (!l->handle)
-     {
-        free(l);
-        return NULL;
-     }
-   l->load2 = dlsym(l->handle, "load2");
-   if (!l->load2)
-      l->load = dlsym(l->handle, "load");
-   l->save = dlsym(l->handle, "save");
-   l_formats = dlsym(l->handle, "formats");
+      goto bail;
 
-   /* each loader must provide formats() and at least load() or save() */
-   if (!l_formats || !(l->load2 || l->load || l->save))
+   l->module = m = dlsym(l->handle, "loader");
+   if (!l->module)
+      goto bail;
+
+   /* Check version and that we have at least load() or save() */
+   if (m->ldr_version != IMLIB2_LOADER_VERSION ||
+       !m->formats || m->num_formats <= 0 || !(m->load || m->save))
      {
         dlclose(l->handle);
-        free(l);
-        return NULL;
+        goto bail;
      }
-   l_formats(l);
+
    l->file = strdup(file);
-   l->next = NULL;
+   l->name = m->formats[0];
 
    l->next = loaders;
    loaders = l;
 
    return l;
+
+ bail:
+   free(l);
+   return NULL;
 }
 
 /* fre the struct for a loader and close its dlopen'd handle */
@@ -154,14 +196,6 @@ __imlib_ConsumeLoader(ImlibLoader * l)
    free(l->file);
    if (l->handle)
       dlclose(l->handle);
-   if (l->formats)
-     {
-        int                 i;
-
-        for (i = 0; i < l->num_formats; i++)
-           free(l->formats[i]);
-        free(l->formats);
-     }
    free(l);
 }
 
@@ -191,7 +225,7 @@ __imlib_LoadAllLoaders(void)
    DP("%s\n", __func__);
 
    /* list all the loaders imlib can find */
-   list = __imlib_ListModules(__imlib_PathToLoaders(), &num);
+   list = __imlib_ModulesList(__imlib_PathToLoaders(), &num);
    /* no loaders? well don't load anything */
    if (!list)
       return;
@@ -224,7 +258,7 @@ __imlib_LookupKnownLoader(const char *format)
    ImlibLoader        *l;
    unsigned int        i;
    const char         *const *exts;
-   char                nbuf[4096];
+   char               *dso;
 
    kl = NULL;
    for (i = 0; i < ARRAY_SIZE(loaders_known); i++)
@@ -242,18 +276,25 @@ __imlib_LookupKnownLoader(const char *format)
    l = NULL;
    if (kl)
      {
-        snprintf(nbuf, sizeof(nbuf), "%s/%s.so", __imlib_PathToLoaders(),
-                 kl->dso);
-        l = __imlib_ProduceLoader(nbuf);
+        dso = __imlib_ModuleFind(__imlib_PathToLoaders(), kl->dso);
+        l = __imlib_ProduceLoader(dso);
+        free(dso);
      }
    DP("%s: '%s' -> '%s': %p\n", __func__, format, kl ? kl->dso : "-", l);
    return l;
 }
 
+static int
+_loader_ok_for(const ImlibLoader * l, int for_save)
+{
+   return (for_save && l->module->save) || (!for_save && l->module->load);
+}
+
 static ImlibLoader *
 __imlib_LookupLoadedLoader(const char *format, int for_save)
 {
    ImlibLoader        *l;
+   ImlibLoaderModule  *m;
 
    DP("%s: fmt='%s'\n", __func__, format);
 
@@ -273,15 +314,16 @@ __imlib_LookupLoadedLoader(const char *format, int for_save)
      {
         int                 i;
 
+        m = l->module;
+
         /* go through all the formats that loader supports */
-        for (i = 0; i < l->num_formats; i++)
+        for (i = 0; i < m->num_formats; i++)
           {
              /* does it match ? */
-             if (strcasecmp(l->formats[i], format) == 0)
+             if (strcasecmp(m->formats[i], format) == 0)
                {
                   /* does it provide the function we need? */
-                  if ((for_save && l->save) ||
-                      (!for_save && (l->load || l->load2)))
+                  if (_loader_ok_for(l, for_save))
                      goto done;
                }
           }
@@ -293,11 +335,14 @@ __imlib_LookupLoadedLoader(const char *format, int for_save)
 }
 
 __EXPORT__ ImlibLoader *
-__imlib_FindBestLoaderForFormat(const char *format, int for_save)
+__imlib_FindBestLoader(const char *file, const char *format, int for_save)
 {
    ImlibLoader        *l;
 
-   DP("%s: fmt='%s'\n", __func__, format);
+   DP("%s: file='%s' fmt='%s'\n", __func__, file, format);
+
+   if (!format)
+      format = __imlib_FileExtension(file);
 
    if (!format || format[0] == '\0')
       return NULL;
@@ -311,7 +356,7 @@ __imlib_FindBestLoaderForFormat(const char *format, int for_save)
      }
 
    l = __imlib_LookupKnownLoader(format);
-   if (l)
+   if (l && _loader_ok_for(l, for_save))
       goto done;
 
    __imlib_LoadAllLoaders();
@@ -322,42 +367,3 @@ __imlib_FindBestLoaderForFormat(const char *format, int for_save)
    DP("%s: fmt='%s': %s\n", __func__, format, l ? l->file : "-");
    return l;
 }
-
-__EXPORT__ ImlibLoader *
-__imlib_FindBestLoaderForFile(const char *file, int for_save)
-{
-   ImlibLoader        *l;
-
-   DP("%s: file='%s'\n", __func__, file);
-
-   l = __imlib_FindBestLoaderForFormat(__imlib_FileExtension(file), for_save);
-
-   return l;
-}
-
-ImlibLoader        *
-__imlib_FindBestLoaderForFileFormat(const char *file, const char *format,
-                                    int for_save)
-{
-   DP("%s: file='%s' ext='%s'\n", __func__, file, format);
-
-   /* if the format is provided ("png" "jpg" etc.) use that */
-   /* otherwise us the file extension */
-   if (format)
-      return __imlib_FindBestLoaderForFormat(format, for_save);
-   else
-      return __imlib_FindBestLoaderForFile(file, for_save);
-}
-
-__EXPORT__ void
-__imlib_LoaderSetFormats(ImlibLoader * l,
-                         const char *const *fmt, unsigned int num)
-{
-   unsigned int        i;
-
-   l->num_formats = num;
-   l->formats = malloc(sizeof(char *) * num);
-
-   for (i = 0; i < num; i++)
-      l->formats[i] = strdup(fmt[i]);
-}
