@@ -1,20 +1,20 @@
 #include "common.h"
 
 #include <assert.h>
+#include <stdlib.h>
+#include <string.h>
 
 #include "asm_c.h"
-#include "blend.h"
-#include "colormod.h"
 #include "image.h"
 #include "scale.h"
 
 /*\ NB: If you change this, don't forget asm_scale.S \*/
 struct _imlib_scale_info {
    int                *xpoints;
-   DATA32            **ypoints;
+   uint32_t          **ypoints;
    int                *xapoints, *yapoints;
    int                 xup_yup;
-   DATA32             *pix_assert;
+   uint32_t           *pix_assert;
 };
 
 #define INV_XAP                   (256 - xapoints[x])
@@ -22,10 +22,10 @@ struct _imlib_scale_info {
 #define INV_YAP                   (256 - yapoints[dyy + y])
 #define YAP                       (yapoints[dyy + y])
 
-static DATA32     **
-__imlib_CalcYPoints(DATA32 * src, int sw, int sh, int dh, int b1, int b2)
+static uint32_t   **
+__imlib_CalcYPoints(uint32_t * src, int sw, int sh, int dh, int b1, int b2)
 {
-   DATA32            **p;
+   uint32_t          **p;
    int                 i, j = 0;
    int                 val, inc, rv = 0;
 
@@ -35,7 +35,9 @@ __imlib_CalcYPoints(DATA32 * src, int sw, int sh, int dh, int b1, int b2)
         rv = 1;
      }
 
-   p = malloc((dh + 1) * sizeof(DATA32 *));
+   p = malloc((dh + 1) * sizeof(uint32_t *));
+   if (!p)
+      return NULL;
 
    val = MIN(sh, dh);
    inc = b1 + b2;
@@ -73,7 +75,7 @@ __imlib_CalcYPoints(DATA32 * src, int sw, int sh, int dh, int b1, int b2)
    if (rv)
       for (i = dh / 2; --i >= 0;)
         {
-           DATA32             *tmp = p[i];
+           uint32_t           *tmp = p[i];
 
            p[i] = p[dh - i - 1];
            p[dh - i - 1] = tmp;
@@ -94,6 +96,8 @@ __imlib_CalcXPoints(int sw, int dw, int b1, int b2)
      }
 
    p = malloc((dw + 1) * sizeof(int));
+   if (!p)
+      return NULL;
 
    val = MIN(sw, dw);
    inc = b1 + b2;
@@ -152,6 +156,8 @@ __imlib_CalcApoints(int s, int d, int b1, int b2, int up)
      }
 
    p = malloc(d * sizeof(int));
+   if (!p)
+      return NULL;
 
    val = MIN(s, d);
    inc = b1 + b2;
@@ -281,12 +287,12 @@ __imlib_CalcScaleInfo(ImlibImage * im, int sw, int sh, int dw, int dh, char aa)
 
 /* scale by pixel sampling only */
 void
-__imlib_ScaleSampleRGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
+__imlib_ScaleSampleRGBA(ImlibScaleInfo * isi, uint32_t * dest, int dxx, int dyy,
                         int dx, int dy, int dw, int dh, int dow)
 {
-   DATA32             *sptr, *dptr;
+   uint32_t           *sptr, *dptr;
    int                 x, y, end;
-   DATA32            **ypoints = isi->ypoints;
+   uint32_t          **ypoints = isi->ypoints;
    int                *xpoints = isi->xpoints;
 
    /* whats the last pixel on the line so we stop there */
@@ -308,12 +314,12 @@ __imlib_ScaleSampleRGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
 
 /* scale by area sampling */
 void
-__imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
+__imlib_ScaleAARGBA(ImlibScaleInfo * isi, uint32_t * dest, int dxx, int dyy,
                     int dx, int dy, int dw, int dh, int dow, int sow)
 {
-   DATA32             *sptr, *dptr;
+   uint32_t           *sptr, *dptr;
    int                 x, y, end;
-   DATA32            **ypoints;
+   uint32_t          **ypoints;
    int                *xpoints;
    int                *xapoints;
    int                *yapoints;
@@ -347,7 +353,7 @@ __imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
                     {
                        int                 r, g, b, a;
                        int                 rr, gg, bb, aa;
-                       DATA32             *pix;
+                       uint32_t           *pix;
 
                        if (XAP > 0)
                          {
@@ -402,7 +408,7 @@ __imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
                   for (x = dxx; x < end; x++)
                     {
                        int                 r, g, b, a;
-                       DATA32             *pix;
+                       uint32_t           *pix;
 
                        if (XAP > 0)
                          {
@@ -434,7 +440,7 @@ __imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
      {
         /*\ 'Correct' version, with math units prepared for MMXification \ */
         int                 Cy, j;
-        DATA32             *pix;
+        uint32_t           *pix;
         int                 r, g, b, a, rr, gg, bb, aa;
         int                 yap;
 
@@ -532,7 +538,7 @@ __imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
                     {
                        int                 r = 0, g = 0, b = 0, a = 0;
                        int                 rr = 0, gg = 0, bb = 0, aa = 0;
-                       DATA32             *pix;
+                       uint32_t           *pix;
 
                        if (XAP > 0)
                          {
@@ -584,7 +590,7 @@ __imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
                     {
                        int                 r = 0, g = 0, b = 0, a = 0;
                        int                 count;
-                       DATA32             *pix;
+                       uint32_t           *pix;
 
                        if (XAP > 0)
                          {
@@ -617,7 +623,7 @@ __imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
      {
         /*\ 'Correct' version, with math units prepared for MMXification \ */
         int                 Cx, j;
-        DATA32             *pix;
+        uint32_t           *pix;
         int                 r, g, b, a, rr, gg, bb, aa;
         int                 xap;
 
@@ -712,7 +718,7 @@ __imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
                        int                 r = 0, g = 0, b = 0, a = 0;
                        int                 rr = 0, gg = 0, bb = 0, aa = 0;
                        int                 xap;
-                       DATA32             *pix;
+                       uint32_t           *pix;
 
                        xap = xpoints[x + 1] - xpoints[x];
                        if (xap > 1)
@@ -769,7 +775,7 @@ __imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
                     {
                        int                 r = 0, g = 0, b = 0, a = 0;
                        int                 xap;
-                       DATA32             *pix;
+                       uint32_t           *pix;
 
                        xap = xpoints[x + 1] - xpoints[x];
                        if (xap > 1)
@@ -805,7 +811,7 @@ __imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
          * |*|  psllw (16 - d), %mmb; pmulh %mmc, %mmb
          * \ */
         int                 Cx, Cy, i, j;
-        DATA32             *pix;
+        uint32_t           *pix;
         int                 a, r, g, b, ax, rx, gx, bx;
         int                 xap, yap;
 
@@ -920,7 +926,7 @@ __imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
 #else
      {
         int                 count;
-        DATA32             *pix;
+        uint32_t           *pix;
         int                 a, r, g, b;
 
         /* go through every scanline in the output buffer */
@@ -969,12 +975,12 @@ __imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
 
 /* scale by area sampling - IGNORE the ALPHA byte*/
 void
-__imlib_ScaleAARGB(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
+__imlib_ScaleAARGB(ImlibScaleInfo * isi, uint32_t * dest, int dxx, int dyy,
                    int dx, int dy, int dw, int dh, int dow, int sow)
 {
-   DATA32             *sptr, *dptr;
+   uint32_t           *sptr, *dptr;
    int                 x, y, end;
-   DATA32            **ypoints;
+   uint32_t          **ypoints;
    int                *xpoints;
    int                *xapoints;
    int                *yapoints;
@@ -1008,7 +1014,7 @@ __imlib_ScaleAARGB(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
                     {
                        int                 r = 0, g = 0, b = 0;
                        int                 rr = 0, gg = 0, bb = 0;
-                       DATA32             *pix;
+                       uint32_t           *pix;
 
                        if (XAP > 0)
                          {
@@ -1055,7 +1061,7 @@ __imlib_ScaleAARGB(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
                   for (x = dxx; x < end; x++)
                     {
                        int                 r = 0, g = 0, b = 0;
-                       DATA32             *pix;
+                       uint32_t           *pix;
 
                        if (XAP > 0)
                          {
@@ -1084,7 +1090,7 @@ __imlib_ScaleAARGB(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
      {
         /*\ 'Correct' version, with math units prepared for MMXification \ */
         int                 Cy, j;
-        DATA32             *pix;
+        uint32_t           *pix;
         int                 r, g, b, rr, gg, bb;
         int                 yap;
 
@@ -1171,7 +1177,7 @@ __imlib_ScaleAARGB(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
                     {
                        int                 r = 0, g = 0, b = 0;
                        int                 rr = 0, gg = 0, bb = 0;
-                       DATA32             *pix;
+                       uint32_t           *pix;
 
                        if (XAP > 0)
                          {
@@ -1216,7 +1222,7 @@ __imlib_ScaleAARGB(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
                   for (x = dxx; x < end; x++)
                     {
                        int                 r = 0, g = 0, b = 0;
-                       DATA32             *pix;
+                       uint32_t           *pix;
 
                        if (XAP > 0)
                          {
@@ -1246,7 +1252,7 @@ __imlib_ScaleAARGB(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
      {
         /*\ 'Correct' version, with math units prepared for MMXification \ */
         int                 Cx, j;
-        DATA32             *pix;
+        uint32_t           *pix;
         int                 r, g, b, rr, gg, bb;
         int                 xap;
 
@@ -1330,7 +1336,7 @@ __imlib_ScaleAARGB(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
                        int                 r = 0, g = 0, b = 0;
                        int                 rr = 0, gg = 0, bb = 0;
                        int                 xap;
-                       DATA32             *pix;
+                       uint32_t           *pix;
 
                        xap = xpoints[x + 1] - xpoints[x];
                        if (xap > 1)
@@ -1380,7 +1386,7 @@ __imlib_ScaleAARGB(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
                     {
                        int                 r = 0, g = 0, b = 0;
                        int                 xap;
-                       DATA32             *pix;
+                       uint32_t           *pix;
 
                        xap = xpoints[x + 1] - xpoints[x];
                        if (xap > 1)
@@ -1411,7 +1417,7 @@ __imlib_ScaleAARGB(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
      {
         /*\ 'Correct' version, with math units prepared for MMXification \ */
         int                 Cx, Cy, i, j;
-        DATA32             *pix;
+        uint32_t           *pix;
         int                 r, g, b, rx, gx, bx;
         int                 xap, yap;
 
@@ -1513,7 +1519,7 @@ __imlib_ScaleAARGB(ImlibScaleInfo * isi, DATA32 * dest, int dxx, int dyy,
 #else
      {
         int                 count;
-        DATA32             *pix;
+        uint32_t           *pix;
         int                 r, g, b;
 
         /* go through every scanline in the output buffer */
