@@ -1,20 +1,20 @@
 #ifndef __SPAN
 #define __SPAN 1
 
-#include "common.h"
+#include "types.h"
 
-typedef void        (*ImlibPointDrawFunction)(DATA32, DATA32 *);
+typedef void        (*ImlibPointDrawFunction)(uint32_t, uint32_t *);
 
 ImlibPointDrawFunction
 __imlib_GetPointDrawFunction(ImlibOp op, char dst_alpha, char blend);
 
-typedef void        (*ImlibSpanDrawFunction)(DATA32, DATA32 *, int);
+typedef void        (*ImlibSpanDrawFunction)(uint32_t, uint32_t *, int);
 
 ImlibSpanDrawFunction
 __imlib_GetSpanDrawFunction(ImlibOp op, char dst_alpha, char blend);
 
-typedef void        (*ImlibShapedSpanDrawFunction)(DATA8 *, DATA32, DATA32 *,
-                                                   int);
+typedef void        (*ImlibShapedSpanDrawFunction)(uint8_t *, uint32_t,
+                                                   uint32_t *, int);
 
 ImlibShapedSpanDrawFunction
 __imlib_GetShapedSpanDrawFunction(ImlibOp op, char dst_alpha, char blend);
