@@ -11,7 +11,7 @@
 #include "x11_ximage.h"
 
 static char         _x_err = 0;
-static DATA8        rtab[256], gtab[256], btab[256];
+static uint8_t      rtab[256], gtab[256], btab[256];
 
 static int
 Tmp_HandleXError(Display * d, XErrorEvent * ev)
@@ -21,16 +21,18 @@ Tmp_HandleXError(Display * d, XErrorEvent * ev)
 }
 
 void
-__imlib_GrabXImageToRGBA(DATA32 * data,
+__imlib_GrabXImageToRGBA(uint32_t * data,
                          int x_dst, int y_dst, int w_dst, int h_dst,
                          Display * d, XImage * xim, XImage * mxim, Visual * v,
                          int depth,
                          int x_src, int y_src, int w_src, int h_src, int grab)
 {
    int                 x, y, inx, iny;
-   const DATA32       *src;
-   DATA32             *ptr;
+   const uint32_t     *src;
+   const uint16_t     *s16;
+   uint32_t           *ptr;
    int                 pixel;
+   uint16_t            p16;
    int                 bgr = 0;
 
    if (!data)
@@ -160,73 +162,44 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
 #undef RMSK
 #undef GMSK
 #undef BMSK
-#undef R1SH
-#undef G1SH
-#undef B1SH
-#undef R2SH
-#undef G2SH
-#undef B2SH
-#undef P1
-#undef P2
-#define MP(x, y) ((XGetPixel(mxim, (x), (y))) ? 0xff000000 : 0)
-#define RMSK  0xf80000
-#define GMSK  0x00fc00
-#define BMSK  0x0000f8
-#define R1SH(p)  ((p) << 8)
-#define G1SH(p)  ((p) << 5)
-#define B1SH(p)  ((p) << 3)
-#define R2SH(p)  ((p) >> 8)
-#define G2SH(p)  ((p) >> 11)
-#define B2SH(p)  ((p) >> 13)
-#define P1(p) (R1SH(p) & RMSK) | (G1SH(p) & GMSK) | (B1SH(p) & BMSK)
-#define P2(p) (R2SH(p) & RMSK) | (G2SH(p) & GMSK) | (B2SH(p) & BMSK)
+#undef RSH
+#undef GSH
+#undef BSH
+#define MP(x, y) ((XGetPixel(mxim, (x), (y))) ? 0xff: 0)
+#define RMSK  0x1f
+#define GMSK  0x3f
+#define BMSK  0x1f
+#define RSH(p)  ((p) >> 11)
+#define GSH(p)  ((p) >>  5)
+#define BSH(p)  ((p) >>  0)
+#define RVAL(p) (255 * (RSH(p) & RMSK)) / RMSK
+#define GVAL(p) (255 * (GSH(p) & GMSK)) / GMSK
+#define BVAL(p) (255 * (BSH(p) & BMSK)) / BMSK
         if (mxim)
           {
              for (y = 0; y < h_src; y++)
                {
-                  src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                  s16 = (uint16_t *) (xim->data + (xim->bytes_per_line * y));
                   ptr = data + ((y + iny) * w_dst) + inx;
-                  for (x = 0; x < (w_src - 1); x += 2)
+                  for (x = 0; x < w_src; x++)
                     {
-#ifdef WORDS_BIGENDIAN
-                       *ptr++ = MP(x + 1, y) | P2(*src);
-                       *ptr++ = MP(x, y) | P1(*src);
-#else
-                       *ptr++ = MP(x, y) | P1(*src);
-                       *ptr++ = MP(x + 1, y) | P2(*src);
-#endif
-                       src++;
-                    }
-                  if (x == (w_src - 1))
-                    {
-                       pixel = XGetPixel(xim, x, y);
-                       *ptr++ = MP(x, y) | P1(pixel);
+                       p16 = *s16++;
+                       *ptr++ =
+                          PIXEL_ARGB(MP(x, y), RVAL(p16), GVAL(p16), BVAL(p16));
                     }
                }
           }
-#undef MP
-#define MP(x, y) (0xff000000)
         else
           {
              for (y = 0; y < h_src; y++)
                {
-                  src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                  s16 = (uint16_t *) (xim->data + (xim->bytes_per_line * y));
                   ptr = data + ((y + iny) * w_dst) + inx;
-                  for (x = 0; x < (w_src - 1); x += 2)
+                  for (x = 0; x < w_src; x++)
                     {
-#ifdef WORDS_BIGENDIAN
-                       *ptr++ = MP(x + 1, y) | P2(*src);
-                       *ptr++ = MP(x, y) | P1(*src);
-#else
-                       *ptr++ = MP(x, y) | P1(*src);
-                       *ptr++ = MP(x + 1, y) | P2(*src);
-#endif
-                       src++;
-                    }
-                  if (x == (w_src - 1))
-                    {
-                       pixel = XGetPixel(xim, x, y);
-                       *ptr++ = MP(x, y) | P1(pixel);
+                       p16 = *s16++;
+                       *ptr++ =
+                          PIXEL_ARGB(0xff, RVAL(p16), GVAL(p16), BVAL(p16));
                     }
                }
           }
@@ -236,73 +209,44 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
 #undef RMSK
 #undef GMSK
 #undef BMSK
-#undef R1SH
-#undef G1SH
-#undef B1SH
-#undef R2SH
-#undef G2SH
-#undef B2SH
-#undef P1
-#undef P2
-#define MP(x, y) ((XGetPixel(mxim, (x), (y))) ? 0xff000000 : 0)
-#define RMSK  0xf80000
-#define GMSK  0x00f800
-#define BMSK  0x0000f8
-#define R1SH(p)  ((p) << 9)
-#define G1SH(p)  ((p) << 6)
-#define B1SH(p)  ((p) << 3)
-#define R2SH(p)  ((p) >> 7)
-#define G2SH(p)  ((p) >> 10)
-#define B2SH(p)  ((p) >> 13)
-#define P1(p) (R1SH(p) & RMSK) | (G1SH(p) & GMSK) | (B1SH(p) & BMSK)
-#define P2(p) (R2SH(p) & RMSK) | (G2SH(p) & GMSK) | (B2SH(p) & BMSK)
+#undef RSH
+#undef GSH
+#undef BSH
+#define MP(x, y) ((XGetPixel(mxim, (x), (y))) ? 0xff: 0)
+#define RMSK  0x1f
+#define GMSK  0x1f
+#define BMSK  0x1f
+#define RSH(p)  ((p) >> 10)
+#define GSH(p)  ((p) >>  5)
+#define BSH(p)  ((p) >>  0)
+#define RVAL(p) (255 * (RSH(p) & RMSK)) / RMSK
+#define GVAL(p) (255 * (GSH(p) & GMSK)) / GMSK
+#define BVAL(p) (255 * (BSH(p) & BMSK)) / BMSK
         if (mxim)
           {
              for (y = 0; y < h_src; y++)
                {
-                  src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                  s16 = (uint16_t *) (xim->data + (xim->bytes_per_line * y));
                   ptr = data + ((y + iny) * w_dst) + inx;
-                  for (x = 0; x < (w_src - 1); x += 2)
+                  for (x = 0; x < w_src; x++)
                     {
-#ifdef WORDS_BIGENDIAN
-                       *ptr++ = MP(x + 1, y) | P2(*src);
-                       *ptr++ = MP(x, y) | P1(*src);
-#else
-                       *ptr++ = MP(x, y) | P1(*src);
-                       *ptr++ = MP(x + 1, y) | P2(*src);
-#endif
-                       src++;
-                    }
-                  if (x == (w_src - 1))
-                    {
-                       pixel = XGetPixel(xim, x, y);
-                       *ptr++ = MP(x, y) | P1(pixel);
+                       p16 = *s16++;
+                       *ptr++ =
+                          PIXEL_ARGB(MP(x, y), RVAL(p16), GVAL(p16), BVAL(p16));
                     }
                }
           }
-#undef MP
-#define MP(x, y) (0xff000000)
         else
           {
              for (y = 0; y < h_src; y++)
                {
-                  src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                  s16 = (uint16_t *) (xim->data + (xim->bytes_per_line * y));
                   ptr = data + ((y + iny) * w_dst) + inx;
-                  for (x = 0; x < (w_src - 1); x += 2)
+                  for (x = 0; x < w_src; x++)
                     {
-#ifdef WORDS_BIGENDIAN
-                       *ptr++ = MP(x + 1, y) | P2(*src);
-                       *ptr++ = MP(x, y) | P1(*src);
-#else
-                       *ptr++ = MP(x, y) | P1(*src);
-                       *ptr++ = MP(x + 1, y) | P2(*src);
-#endif
-                       src++;
-                    }
-                  if (x == (w_src - 1))
-                    {
-                       pixel = XGetPixel(xim, x, y);
-                       *ptr++ = MP(x, y) | P1(pixel);
+                       p16 = *s16++;
+                       *ptr++ =
+                          PIXEL_ARGB(0xff, RVAL(p16), GVAL(p16), BVAL(p16));
                     }
                }
           }
@@ -380,7 +324,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
                {
                   for (y = 0; y < h_src; y++)
                     {
-                       src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                       src =
+                          (uint32_t *) (xim->data + (xim->bytes_per_line * y));
                        ptr = data + ((y + iny) * w_dst) + inx;
                        for (x = 0; x < w_src; x++)
                          {
@@ -397,7 +342,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
                {
                   for (y = 0; y < h_src; y++)
                     {
-                       src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                       src =
+                          (uint32_t *) (xim->data + (xim->bytes_per_line * y));
                        ptr = data + ((y + iny) * w_dst) + inx;
                        for (x = 0; x < w_src; x++)
                          {
@@ -415,7 +361,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
                {
                   for (y = 0; y < h_src; y++)
                     {
-                       src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                       src =
+                          (uint32_t *) (xim->data + (xim->bytes_per_line * y));
                        ptr = data + ((y + iny) * w_dst) + inx;
                        for (x = 0; x < w_src; x++)
                          {
@@ -431,7 +378,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
                {
                   for (y = 0; y < h_src; y++)
                     {
-                       src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                       src =
+                          (uint32_t *) (xim->data + (xim->bytes_per_line * y));
                        ptr = data + ((y + iny) * w_dst) + inx;
                        for (x = 0; x < w_src; x++)
                          {
@@ -449,7 +397,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
                {
                   for (y = 0; y < h_src; y++)
                     {
-                       src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                       src =
+                          (uint32_t *) (xim->data + (xim->bytes_per_line * y));
                        ptr = data + ((y + iny) * w_dst) + inx;
                        for (x = 0; x < w_src; x++)
                          {
@@ -467,7 +416,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
                {
                   for (y = 0; y < h_src; y++)
                     {
-                       src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                       src =
+                          (uint32_t *) (xim->data + (xim->bytes_per_line * y));
                        ptr = data + ((y + iny) * w_dst) + inx;
                        for (x = 0; x < w_src; x++)
                          {
@@ -486,7 +436,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
                {
                   for (y = 0; y < h_src; y++)
                     {
-                       src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                       src =
+                          (uint32_t *) (xim->data + (xim->bytes_per_line * y));
                        ptr = data + ((y + iny) * w_dst) + inx;
                        for (x = 0; x < w_src; x++)
                          {
@@ -504,7 +455,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
                {
                   for (y = 0; y < h_src; y++)
                     {
-                       src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                       src =
+                          (uint32_t *) (xim->data + (xim->bytes_per_line * y));
                        ptr = data + ((y + iny) * w_dst) + inx;
                        for (x = 0; x < w_src; x++)
                          {
@@ -525,7 +477,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
                {
                   for (y = 0; y < h_src; y++)
                     {
-                       src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                       src =
+                          (uint32_t *) (xim->data + (xim->bytes_per_line * y));
                        ptr = data + ((y + iny) * w_dst) + inx;
                        for (x = 0; x < w_src; x++)
                          {
@@ -541,7 +494,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
                {
                   for (y = 0; y < h_src; y++)
                     {
-                       src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                       src =
+                          (uint32_t *) (xim->data + (xim->bytes_per_line * y));
                        ptr = data + ((y + iny) * w_dst) + inx;
                        for (x = 0; x < w_src; x++)
                          {
@@ -557,7 +511,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
                {
                   for (y = 0; y < h_src; y++)
                     {
-                       src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                       src =
+                          (uint32_t *) (xim->data + (xim->bytes_per_line * y));
                        ptr = data + ((y + iny) * w_dst) + inx;
                        for (x = 0; x < w_src; x++)
                          {
@@ -572,7 +527,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data,
                {
                   for (y = 0; y < h_src; y++)
                     {
-                       src = (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                       src =
+                          (uint32_t *) (xim->data + (xim->bytes_per_line * y));
                        ptr = data + ((y + iny) * w_dst) + inx;
                        for (x = 0; x < w_src; x++)
                          {
@@ -634,7 +590,7 @@ _WindowGetShapeMask(Display * d, Window p,
 }
 
 int
-__imlib_GrabDrawableToRGBA(DATA32 * data, int x_dst, int y_dst, int w_dst,
+__imlib_GrabDrawableToRGBA(uint32_t * data, int x_dst, int y_dst, int w_dst,
                            int h_dst, Display * d, Drawable p, Pixmap m_,
                            Visual * v, Colormap cm, int depth, int x_src,
                            int y_src, int w_src, int h_src, char *pdomask,
@@ -864,7 +820,7 @@ __imlib_GrabDrawableToRGBA(DATA32 * data, int x_dst, int y_dst, int w_dst,
 }
 
 int
-__imlib_GrabDrawableScaledToRGBA(DATA32 * data, int nu_x_dst, int nu_y_dst,
+__imlib_GrabDrawableScaledToRGBA(uint32_t * data, int nu_x_dst, int nu_y_dst,
                                  int w_dst, int h_dst,
                                  Display * d, Drawable p, Pixmap m_,
                                  Visual * v, Colormap cm, int depth,
