@@ -1,8 +1,32 @@
 #include "common.h"
 
+#include <stdlib.h>
+#include <string.h>
 #include <X11/Xlib.h>
 
+#include "blend.h"
+#include "colormod.h"
+#include "image.h"
 #include "x11_pixmap.h"
+#include "x11_rend.h"
+
+typedef struct _ImlibImagePixmap {
+   int                 w, h;
+   Pixmap              pixmap, mask;
+   Display            *display;
+   Visual             *visual;
+   int                 depth;
+   int                 source_x, source_y, source_w, source_h;
+   Colormap            colormap;
+   char                antialias, hi_quality, dither_mask;
+   ImlibBorder         border;
+   ImlibImage         *image;
+   char               *file;
+   char                dirty;
+   int                 references;
+   uint64_t            modification_count;
+   struct _ImlibImagePixmap *next;
+} ImlibImagePixmap;
 
 static ImlibImagePixmap *pixmaps = NULL;
 
@@ -34,11 +58,11 @@ __imlib_ConsumeImagePixmap(ImlibImagePixmap * ip)
    free(ip);
 }
 
-ImlibImagePixmap   *
+static ImlibImagePixmap *
 __imlib_FindCachedImagePixmap(ImlibImage * im, int w, int h, Display * d,
                               Visual * v, int depth, int sx, int sy, int sw,
                               int sh, Colormap cm, char aa, char hiq,
-                              char dmask, DATABIG modification_count)
+                              char dmask, uint64_t modification_count)
 {
    ImlibImagePixmap   *ip, *ip_prev;
 
@@ -73,13 +97,13 @@ __imlib_FindCachedImagePixmap(ImlibImage * im, int w, int h, Display * d,
 }
 
 /* add a pixmap cahce struct to the pixmap cache (at the start of course */
-ImlibImagePixmap   *
+static ImlibImagePixmap *
 __imlib_AddImagePixmapToCache(ImlibImage * im, Pixmap pmap, Pixmap mask,
                               int w, int h,
                               Display * d, Visual * v, int depth,
                               int sx, int sy, int sw, int sh,
                               Colormap cm, char aa, char hiq, char dmask,
-                              DATABIG modification_count)
+                              uint64_t modification_count)
 {
    ImlibImagePixmap   *ip;
 
@@ -272,7 +296,8 @@ __imlib_PixmapCacheSize(void)
         if (ip->references == 0)
           {
              /* if the image is invalid */
-             if ((ip->dirty) || ((ip->image) && (!(IMAGE_IS_VALID(ip->image)))))
+             if (ip->dirty ||
+                 (ip->image && IM_FLAG_ISSET(ip->image, F_INVALID)))
                {
                   __imlib_RemoveImagePixmapFromCache(ip);
                   __imlib_ConsumeImagePixmap(ip);
@@ -300,3 +325,61 @@ __imlib_PixmapCacheSize(void)
 
    return current_cache;
 }
+
+char
+__imlib_CreatePixmapsForImage(Display * d, Drawable w, Visual * v, int depth,
+                              Colormap cm, ImlibImage * im, Pixmap * p,
+                              Mask * m, int sx, int sy, int sw, int sh, int dw,
+                              int dh, char antialias, char hiq,
+                              char dither_mask, int mat,
+                              ImlibColorModifier * cmod)
+{
+   ImlibImagePixmap   *ip = NULL;
+   Pixmap              pmap = 0;
+   Pixmap              mask = 0;
+   long long           mod_count = 0;
+
+   if (cmod)
+      mod_count = cmod->modification_count;
+   ip = __imlib_FindCachedImagePixmap(im, dw, dh, d, v, depth, sx, sy,
+                                      sw, sh, cm, antialias, hiq, dither_mask,
+                                      mod_count);
+   if (ip)
+     {
+        if (p)
+           *p = ip->pixmap;
+        if (m)
+           *m = ip->mask;
+        ip->references++;
+#ifdef DEBUG_CACHE
+        fprintf(stderr,
+                "[Imlib2]  Match found in cache.  Reference count is %d, pixmap 0x%08lx, mask 0x%08lx\n",
+                ip->references, ip->pixmap, ip->mask);
+#endif
+        return 2;
+     }
+   if (p)
+     {
+        pmap = XCreatePixmap(d, w, dw, dh, depth);
+        *p = pmap;
+     }
+   if (m)
+     {
+        if (im->has_alpha)
+           mask = XCreatePixmap(d, w, dw, dh, 1);
+        *m = mask;
+     }
+   __imlib_RenderImage(d, im, pmap, mask, v, cm, depth, sx, sy, sw, sh, 0, 0,
+                       dw, dh, antialias, hiq, 0, dither_mask, mat, cmod,
+                       OP_COPY);
+   ip = __imlib_AddImagePixmapToCache(im, pmap, mask,
+                                      dw, dh, d, v, depth, sx, sy,
+                                      sw, sh, cm, antialias, hiq, dither_mask,
+                                      mod_count);
+#ifdef DEBUG_CACHE
+   fprintf(stderr,
+           "[Imlib2]  Created pixmap.  Reference count is %d, pixmap 0x%08lx, mask 0x%08lx\n",
+           ip->references, ip->pixmap, ip->mask);
+#endif
+   return 1;
+}
