@@ -1,5 +1,7 @@
 #include "common.h"
 
+#include <stdlib.h>
+#include <string.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/extensions/XShm.h>
@@ -19,9 +21,9 @@
 /* size of the lines per segment we scale / render at a time */
 #define LINESIZE 16
 
-DATA32
+uint32_t
 __imlib_RenderGetPixel(Display * d, Drawable w, Visual * v, Colormap cm,
-                       int depth, DATA8 r, DATA8 g, DATA8 b)
+                       int depth, uint8_t r, uint8_t g, uint8_t b)
 {
    Context            *ct;
 
@@ -31,35 +33,35 @@ __imlib_RenderGetPixel(Display * d, Drawable w, Visual * v, Colormap cm,
      {
         switch (ct->palette_type)
           {
-          case 0:              /* 332 */
+          case PAL_TYPE_332:
              return ct->palette[((r >> 0) & 0xe0) |
                                 ((g >> 3) & 0x1b) | ((b >> 6) & 0x02)];
              break;
-          case 1:              /* 232 */
+          case PAL_TYPE_232:
              return ct->palette[((r >> 0) & 0xe0) |
                                 ((g >> 3) & 0x1b) | ((b >> 6) & 0x02)];
              break;
-          case 2:              /* 222 */
+          case PAL_TYPE_222:
              return ct->palette[((r >> 0) & 0xe0) |
                                 ((g >> 3) & 0x1b) | ((b >> 6) & 0x02)];
              break;
-          case 3:              /* 221 */
+          case PAL_TYPE_221:
              return ct->palette[((r >> 0) & 0xe0) |
                                 ((g >> 3) & 0x1b) | ((b >> 6) & 0x02)];
              break;
-          case 4:              /* 121 */
+          case PAL_TYPE_121:
              return ct->palette[((r >> 0) & 0xe0) |
                                 ((g >> 3) & 0x1b) | ((b >> 6) & 0x02)];
              break;
-          case 5:              /* 111 */
+          case PAL_TYPE_111:
              return ct->palette[((r >> 0) & 0xe0) |
                                 ((g >> 3) & 0x1b) | ((b >> 6) & 0x02)];
              break;
-          case 6:              /* 1 */
+          case PAL_TYPE_1:
              return ct->palette[((r >> 0) & 0xe0) |
                                 ((g >> 3) & 0x1b) | ((b >> 6) & 0x02)];
              break;
-          case 7:              /* 666 */
+          case PAL_TYPE_666:
              return ct->palette[((int)(((double)r / 255) * 5.0) * 36) +
                                 ((int)(((double)g / 255) * 5.0) * 6) +
                                 ((int)(((double)b / 255) * 5.0))];
@@ -72,7 +74,7 @@ __imlib_RenderGetPixel(Display * d, Drawable w, Visual * v, Colormap cm,
      {
         unsigned int        rm, gm, bm;
         int                 i, rshift = 0, gshift = 0, bshift = 0;
-        DATA32              val;
+        uint32_t            val;
 
         rm = v->red_mask;
         gm = v->green_mask;
@@ -134,7 +136,7 @@ __imlib_RenderGetPixel(Display * d, Drawable w, Visual * v, Colormap cm,
 }
 
 static void
-__imlib_generic_render(DATA32 * src, int jump, int w, int h, int dx, int dy,
+__imlib_generic_render(uint32_t * src, int jump, int w, int h, int dx, int dy,
                        XImage * xim, Visual * v, Context * ct)
 {
    int                 x, y, hh;
@@ -142,7 +144,7 @@ __imlib_generic_render(DATA32 * src, int jump, int w, int h, int dx, int dy,
    unsigned int        rmask, gmask, bmask;
    int                 i, rshift, gshift, bshift;
 
-   static const DATA8  _dither_88[8][8] = {
+   static const uint8_t _dither_88[8][8] = {
       {0, 32, 8, 40, 2, 34, 10, 42},
       {48, 16, 56, 24, 50, 18, 58, 26},
       {12, 44, 4, 36, 14, 46, 6, 38},
@@ -255,7 +257,7 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
 {
    XImage             *xim = NULL, *mxim = NULL;
    Context            *ct;
-   DATA32             *buf = NULL, *pointer = NULL, *back = NULL;
+   uint32_t           *buf = NULL, *pointer = NULL, *back = NULL;
    int                 y, h, hh, jump;
    XGCValues           gcv;
    ImlibScaleInfo     *scaleinfo = NULL;
@@ -265,8 +267,7 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
    ImlibMaskFunction   masker = NULL;
    ImlibBlendFunction  blender = NULL;
 
-   blender = __imlib_GetBlendFunction(op, 1, 0,
-                                      (!(im->flags & F_HAS_ALPHA)), NULL);
+   blender = __imlib_GetBlendFunction(op, 1, 0, !im->has_alpha, NULL);
 
    /* dont do anything if we have a 0 widht or height image to render */
    if ((dw == 0) || (dh == 0))
@@ -314,9 +315,9 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
    dh = abs(dh);
    ct = __imlib_GetContext(d, v, cm, depth);
    __imlib_RGBASetupContext(ct);
-   if ((blend) && (IMAGE_HAS_ALPHA(im)))
+   if (blend && im->has_alpha)
      {
-        back = malloc(dw * dh * sizeof(DATA32));
+        back = malloc(dw * dh * sizeof(uint32_t));
         if (!__imlib_GrabDrawableToRGBA
             (back, 0, 0, dw, dh, d, w, 0, v, cm, depth, dx, dy, dw, dh, 0, 1))
           {
@@ -348,7 +349,7 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
    if (scaleinfo)
      {
         /* allocate a buffer to render scaled RGBA data into */
-        buf = malloc(dw * LINESIZE * sizeof(DATA32));
+        buf = malloc(dw * LINESIZE * sizeof(uint32_t));
         if (!buf)
           {
              __imlib_ConsumeXImage(d, xim);
@@ -379,7 +380,7 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
              /* scale the imagedata for this LINESIZE lines chunk of image data */
              if (antialias)
                {
-                  if (IMAGE_HAS_ALPHA(im))
+                  if (im->has_alpha)
                      __imlib_ScaleAARGBA(scaleinfo, buf, ((sx * dw) / sw),
                                          ((sy * dh) / sh) + y,
                                          0, 0, dw, hh, dw, im->w);
@@ -394,14 +395,14 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
              jump = 0;
              pointer = buf;
              if (cmod)
-                __imlib_DataCmodApply(buf, dw, hh, 0, NULL, cmod);
+                __imlib_DataCmodApply(buf, dw, hh, 0, true, cmod);
           }
         else
           {
              if (cmod)
                {
                   if (!buf)
-                     buf = malloc(im->w * LINESIZE * sizeof(DATA32));
+                     buf = malloc(im->w * LINESIZE * sizeof(uint32_t));
                   if (!buf)
                     {
                        __imlib_ConsumeXImage(d, xim);
@@ -412,8 +413,8 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
                        return;
                     }
                   memcpy(buf, im->data + ((y + sy) * im->w),
-                         im->w * hh * sizeof(DATA32));
-                  __imlib_DataCmodApply(buf, im->w, hh, 0, NULL, cmod);
+                         im->w * hh * sizeof(uint32_t));
+                  __imlib_DataCmodApply(buf, im->w, hh, 0, true, cmod);
                   pointer = buf + sx;
                   jump = im->w - sw;
                }
@@ -433,13 +434,13 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
         /* once scaled... convert chunk to bit depth into XImage bufer */
         if (rgbaer)
            rgbaer(pointer, jump,
-                  ((DATA8 *) xim->data) + (y * (xim->bytes_per_line)),
+                  ((uint8_t *) xim->data) + (y * (xim->bytes_per_line)),
                   xim->bytes_per_line, dw, hh, dx, dy + y);
         else
            __imlib_generic_render(pointer, jump, dw, hh, 0, y, xim, v, ct);
         if (m)
            masker(pointer, jump,
-                  ((DATA8 *) mxim->data) + (y * (mxim->bytes_per_line)),
+                  ((uint8_t *) mxim->data) + (y * (mxim->bytes_per_line)),
                   mxim->bytes_per_line, dw, hh, dx, dy + y, mat);
         h -= LINESIZE;
      }
@@ -564,7 +565,7 @@ __imlib_RenderImageSkewed(Display * d, ImlibImage * im, Drawable w, Drawable m,
    __imlib_GetContext(d, v, cm, depth);
 
    back = __imlib_CreateImage(dw, dh, NULL);
-   back->data = calloc(dw * dh, sizeof(DATA32));
+   back->data = calloc(dw * dh, sizeof(uint32_t));
    __imlib_GrabDrawableToRGBA(back->data, 0, 0, dw, dh, d, w, 0, v, cm,
                               depth, dx1, dy1, dw, dh, 0, 1);
 
