@@ -1,8 +1,12 @@
-#include "filter_common.h"
-#include <string.h>
-#include <math.h>
+#include "config.h"
 #include <Imlib2.h>
 
+#include <math.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "filter_common.h"
+
 #define GET_INT(x, ptr) (((ptr)->type == VAR_PTR) ?	\
 			(x) = (*(int *)(ptr)->data) :	\
 			((ptr)->type == VAR_CHAR) ?	\
@@ -52,11 +56,11 @@ mod_tint(double t[256], double v)
 }
 
 static              Imlib_Image
-colormod(Imlib_Image im, pIFunctionParam par)
+colormod(Imlib_Image im, IFunctionParam * par)
 {
    double              a_d[256], r_d[256], g_d[256], b_d[256];
-   DATA8               a_b[256], r_b[256], g_b[256], b_b[256];
-   pIFunctionParam     ptr;
+   uint8_t             a_b[256], r_b[256], g_b[256], b_b[256];
+   IFunctionParam     *ptr;
    int                 x = 0, y = 0, h, w, i;
    double              v = 0.0;
 
@@ -225,7 +229,7 @@ colormod(Imlib_Image im, pIFunctionParam par)
 }
 
 void
-init(struct imlib_filter_info *info)
+init(ImlibFilterInfo * info)
 {
    static const char  *const filters[] = { "colormod" };
    int                 i = ARRAY_SIZE(filters);
@@ -247,7 +251,7 @@ deinit()
 }
 
 void               *
-exec(char *filter, void *im, pIFunctionParam par)
+exec(char *filter, void *im, IFunctionParam * par)
 {
    if (!strcmp(filter, "colormod"))
       return colormod((Imlib_Image) im, par);
