@@ -1,6 +1,5 @@
-#include "loader_common.h"
-
-#include <sys/mman.h>
+#include "config.h"
+#include "Imlib2_Loader.h"
 
 int
 decompress_load(ImlibImage * im, int load_data, const char *const *pext,
@@ -12,13 +11,12 @@ decompress_load(ImlibImage * im, int load_data, const char *const *pext,
    const char         *s, *p, *q;
    char                tmp[] = "/tmp/imlib2_loader_dec-XXXXXX";
    char               *real_ext;
-   void               *fdata;
 
    rc = LOAD_FAIL;
 
    /* make sure this file ends in a known name and that there's another ext
     * (e.g. "foo.png.bz2") */
-   for (p = s = im->real_file, q = NULL; *s; s++)
+   for (p = s = im->fi->name, q = NULL; *s; s++)
      {
         if (*s != '.' && *s != '/')
            continue;
@@ -43,29 +41,24 @@ decompress_load(ImlibImage * im, int load_data, const char *const *pext,
    if (!(real_ext = strndup(q, p - q - 1)))
       return LOAD_OOM;
 
-   loader = __imlib_FindBestLoaderForFormat(real_ext, 0);
+   loader = __imlib_FindBestLoader(NULL, real_ext, 0);
    free(real_ext);
    if (!loader)
       return rc;
 
-   fdata = mmap(NULL, im->fsize, PROT_READ, MAP_SHARED, fileno(im->fp), 0);
-   if (fdata == MAP_FAILED)
-      return LOAD_BADFILE;
-
    if ((dest = mkstemp(tmp)) < 0)
       QUIT_WITH_RC(LOAD_OOM);
 
-   res = fdec(fdata, im->fsize, dest);
+   res = fdec(im->fi->fdata, im->fi->fsize, dest);
 
    close(dest);
 
    if (res)
-      rc = __imlib_LoadEmbedded(loader, im, tmp, load_data);
+      rc = __imlib_LoadEmbedded(loader, im, load_data, tmp);
 
    unlink(tmp);
 
  quit:
-   munmap(fdata, im->fsize);
 
    return rc;
 }
