@@ -1,6 +1,7 @@
-#include "loader_common.h"
+#include "config.h"
+#include "Imlib2_Loader.h"
 
-#include <sys/mman.h>
+static const char  *const _formats[] = { "argb", "arg" };
 
 static struct {
    const unsigned char *data, *dptr;
@@ -8,7 +9,7 @@ static struct {
 } mdata;
 
 static void
-mm_init(void *src, unsigned int size)
+mm_init(const void *src, unsigned int size)
 {
    mdata.data = mdata.dptr = src;
    mdata.size = size;
@@ -32,13 +33,12 @@ mm_read(void *dst, unsigned int len)
    return 0;
 }
 
-int
-load2(ImlibImage * im, int load_data)
+static int
+_load(ImlibImage * im, int load_data)
 {
    int                 rc;
-   void               *fdata;
    int                 alpha;
-   DATA32             *ptr;
+   uint32_t           *ptr;
    int                 y;
    const char         *fptr, *row;
    unsigned int        size;
@@ -49,16 +49,12 @@ load2(ImlibImage * im, int load_data)
 
    rc = LOAD_FAIL;
 
-   fdata = mmap(NULL, im->fsize, PROT_READ, MAP_SHARED, fileno(im->fp), 0);
-   if (fdata == MAP_FAILED)
-      return LOAD_BADFILE;
-
-   mm_init(fdata, im->fsize);
+   mm_init(im->fi->fdata, im->fi->fsize);
 
    /* header */
 
-   fptr = fdata;
-   size = im->fsize > 31 ? 31 : im->fsize;      /* Look for \n in at most 31 byte */
+   fptr = im->fi->fdata;
+   size = im->fi->fsize > 31 ? 31 : im->fi->fsize;      /* Look for \n in at most 31 byte */
    row = memchr(fptr, '\n', size);
    if (!row)
       goto quit;
@@ -75,7 +71,7 @@ load2(ImlibImage * im, int load_data)
    if (!IMAGE_DIMENSIONS_OK(im->w, im->h))
       goto quit;
 
-   UPDATE_FLAG(im->flags, F_HAS_ALPHA, alpha);
+   im->has_alpha = alpha;
 
    if (!load_data)
       QUIT_WITH_RC(LOAD_SUCCESS);
@@ -106,31 +102,26 @@ load2(ImlibImage * im, int load_data)
    rc = LOAD_SUCCESS;
 
  quit:
-   if (rc <= 0)
-      __imlib_FreeData(im);
-   munmap(fdata, im->fsize);
-
    return rc;
 }
 
-char
-save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
+static int
+_save(ImlibImage * im)
 {
    int                 rc;
    FILE               *f;
-   DATA32             *ptr;
+   uint32_t           *ptr;
    int                 y, alpha = 0;
 
-#ifdef WORDS_BIGENDIAN
-   DATA32             *buf = (DATA32 *) malloc(im->w * 4);
-#endif
-
-   f = fopen(im->real_file, "wb");
+   f = fopen(im->fi->name, "wb");
    if (!f)
       return LOAD_FAIL;
 
-   if (im->flags & F_HAS_ALPHA)
-      alpha = 1;
+#ifdef WORDS_BIGENDIAN
+   uint32_t           *buf = (uint32_t *) malloc(im->w * 4);
+#endif
+
+   alpha = !!im->has_alpha;
 
    fprintf(f, "ARGB %i %i %i\n", im->w, im->h, alpha);
 
@@ -167,9 +158,4 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    return rc;
 }
 
-void
-formats(ImlibLoader * l)
-{
-   static const char  *const list_formats[] = { "argb", "arg" };
-   __imlib_LoaderSetFormats(l, list_formats, ARRAY_SIZE(list_formats));
-}
+IMLIB_LOADER(_formats, _load, _save);
