@@ -7,20 +7,21 @@
  * - Simplify and make secure RLE encoding
  * - Fix 16 and 32 bit depth (old code was incorrect and it's commented)
  */
-#include "loader_common.h"
-
-#include <sys/mman.h>
+#include "config.h"
+#include "Imlib2_Loader.h"
 
 #define DBG_PFX "LDR-bmp"
 #define Dx(fmt...)
 
+static const char  *const _formats[] = { "bmp" };
+
 static struct {
    const unsigned char *data, *dptr;
    unsigned int        size;
 } mdata;
 
 static void
-mm_init(void *src, unsigned int size)
+mm_init(const void *src, unsigned int size)
 {
    mdata.data = mdata.dptr = src;
    mdata.size = size;
@@ -40,42 +41,42 @@ mm_read(void *dst, unsigned int len)
 
 /* The BITMAPFILEHEADER (size 14) */
 typedef struct {
-   DATA8               header[2];
-   DATA8               size[4];
-   DATA8               rsvd1[2];
-   DATA8               rsvd2[2];
-   DATA8               offs[4];
+   uint8_t             header[2];
+   uint8_t             size[4];
+   uint8_t             rsvd1[2];
+   uint8_t             rsvd2[2];
+   uint8_t             offs[4];
 } bfh_t;
 
 /* The BITMAPINFOHEADER */
 typedef union {
-   DATA32              header_size;
+   uint32_t            header_size;
    struct {
       /* BITMAPCOREHEADER (size 12) */
-      DATA32              header_size;
-      DATA16              width;
-      DATA16              height;
-      DATA16              planes;
-      DATA16              bpp;
+      uint32_t            header_size;
+      uint16_t            width;
+      uint16_t            height;
+      uint16_t            planes;
+      uint16_t            bpp;
    } bch;
    struct {
       /* BITMAPINFOHEADER (size 40) */
-      DATA32              header_size;
-      DATA32              width;
-      DATA32              height;
-      DATA16              planes;
-      DATA16              bpp;
-      DATA32              compression;
-      DATA32              size;
-      DATA32              res_hor;
-      DATA32              res_ver;
-      DATA32              colors;
-      DATA32              colors_important;
+      uint32_t            header_size;
+      uint32_t            width;
+      uint32_t            height;
+      uint16_t            planes;
+      uint16_t            bpp;
+      uint32_t            compression;
+      uint32_t            size;
+      uint32_t            res_hor;
+      uint32_t            res_ver;
+      uint32_t            colors;
+      uint32_t            colors_important;
       /* BITMAPV3INFOHEADER (size 56) */
-      DATA32              mask_r;
-      DATA32              mask_g;
-      DATA32              mask_b;
-      DATA32              mask_a;
+      uint32_t            mask_r;
+      uint32_t            mask_g;
+      uint32_t            mask_b;
+      uint32_t            mask_a;
    } bih;
    char                bytes[124];
 } bih_t;
@@ -153,11 +154,10 @@ WriteleLong(FILE * file, unsigned long val)
    return 1;
 }
 
-int
-load2(ImlibImage * im, int load_data)
+static int
+_load(ImlibImage * im, int load_data)
 {
    int                 rc;
-   void               *fdata;
    const unsigned char *fptr;
    bfh_t               bfh;
    unsigned int        bfh_offset;
@@ -167,10 +167,10 @@ load2(ImlibImage * im, int load_data)
    unsigned char       byte = 0, byte1, byte2;
    unsigned int        i, k;
    int                 w, h, x, y, j, l;
-   DATA32             *ptr, pixel;
+   uint32_t           *ptr, pixel;
    const unsigned char *buffer_ptr, *buffer_end, *buffer_end_safe;
    RGBQUAD             rgbQuads[256];
-   DATA32              argbCmap[256];
+   uint32_t            argbCmap[256];
    unsigned int        amask, rmask, gmask, bmask;
    int                 ashift1, rshift1, gshift1, bshift1;
    int                 ashift2, rshift2, gshift2, bshift2;
@@ -178,12 +178,8 @@ load2(ImlibImage * im, int load_data)
 
    rc = LOAD_FAIL;
 
-   fdata = mmap(NULL, im->fsize, PROT_READ, MAP_SHARED, fileno(im->fp), 0);
-   if (fdata == MAP_FAILED)
-      return LOAD_BADFILE;
-
-   fptr = fdata;
-   mm_init(fdata, im->fsize);
+   fptr = im->fi->fdata;
+   mm_init(im->fi->fdata, im->fi->fsize);
 
    /* Load header */
 
@@ -193,7 +189,7 @@ load2(ImlibImage * im, int load_data)
    if (bfh.header[0] != 'B' || bfh.header[1] != 'M')
       goto quit;
 
-   size = im->fsize;
+   size = im->fi->fsize;
 #define WORD_LE_32(p8) (((p8)[3] << 24) | ((p8)[2] << 16) | ((p8)[1] << 8) | (p8)[0])
    bfh_offset = WORD_LE_32(bfh.offs);
 
@@ -258,7 +254,7 @@ load2(ImlibImage * im, int load_data)
         goto quit;
      }
 
-   UPDATE_FLAG(im->flags, F_HAS_ALPHA, amask);
+   im->has_alpha = amask;
 
    imgsize = size - bfh_offset;
    D("w=%3d h=%3d bitcount=%d comp=%d imgsize=%d\n",
@@ -467,7 +463,7 @@ load2(ImlibImage * im, int load_data)
                      x, y, byte1, byte2, byte2 >> 4, byte2 & 0xf);
                   if (byte1)
                     {
-                       DATA32              t1, t2;
+                       uint32_t            t1, t2;
 
                        l = byte1;
                        /* Check for invalid length */
@@ -687,7 +683,7 @@ load2(ImlibImage * im, int load_data)
                {
                   pixel = *(unsigned short *)buffer_ptr;
 
-                  if (im->flags & F_HAS_ALPHA)
+                  if (im->has_alpha)
                      a = SCALE(a, pixel);
                   else
                      a = 0xff;
@@ -736,7 +732,7 @@ load2(ImlibImage * im, int load_data)
                {
                   pixel = *(unsigned int *)buffer_ptr;
 
-                  if (im->flags & F_HAS_ALPHA)
+                  if (im->has_alpha)
                      a = SCALE(a, pixel);
                   else
                      a = 0xff;
@@ -758,22 +754,18 @@ load2(ImlibImage * im, int load_data)
    rc = LOAD_SUCCESS;
 
  quit:
-   if (rc <= 0)
-      __imlib_FreeData(im);
-   munmap(fdata, im->fsize);
-
    return rc;
 }
 
-char
-save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
+static int
+_save(ImlibImage * im)
 {
    int                 rc;
    FILE               *f;
    int                 i, j, pad;
-   DATA32              pixel;
+   uint32_t            pixel;
 
-   f = fopen(im->real_file, "wb");
+   f = fopen(im->fi->name, "wb");
    if (!f)
       return LOAD_FAIL;
 
@@ -819,9 +811,4 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    return rc;
 }
 
-void
-formats(ImlibLoader * l)
-{
-   static const char  *const list_formats[] = { "bmp" };
-   __imlib_LoaderSetFormats(l, list_formats, ARRAY_SIZE(list_formats));
-}
+IMLIB_LOADER(_formats, _load, _save);
