@@ -5,20 +5,22 @@
  * https://en.wikipedia.org/wiki/ICO_(file_format)
  * https://en.wikipedia.org/wiki/BMP_file_format
  */
-#include "loader_common.h"
+#include "config.h"
+#include "Imlib2_Loader.h"
 
 #include <limits.h>
-#include <sys/mman.h>
 
 #define DBG_PFX "LDR-ico"
 
+static const char  *const _formats[] = { "ico" };
+
 static struct {
    const unsigned char *data, *dptr;
    unsigned int        size;
 } mdata;
 
 static void
-mm_init(void *src, unsigned int size)
+mm_init(const void *src, unsigned int size)
 {
    mdata.data = mdata.dptr = src;
    mdata.size = size;
@@ -44,36 +46,36 @@ mm_read(void *dst, unsigned int len)
 
 /* The ICONDIR */
 typedef struct {
-   DATA16              rsvd;
-   DATA16              type;
-   DATA16              icons;
+   uint16_t            rsvd;
+   uint16_t            type;
+   uint16_t            icons;
 } idir_t;
 
 /* The ICONDIRENTRY */
 typedef struct {
-   DATA8               width;
-   DATA8               height;
-   DATA8               colors;
-   DATA8               rsvd;
-   DATA16              planes;
-   DATA16              bpp;
-   DATA32              size;
-   DATA32              offs;
+   uint8_t             width;
+   uint8_t             height;
+   uint8_t             colors;
+   uint8_t             rsvd;
+   uint16_t            planes;
+   uint16_t            bpp;
+   uint32_t            size;
+   uint32_t            offs;
 } ide_t;
 
 /* The BITMAPINFOHEADER */
 typedef struct {
-   DATA32              header_size;
-   DATA32              width;
-   DATA32              height;
-   DATA16              planes;
-   DATA16              bpp;
-   DATA32              compression;
-   DATA32              size;
-   DATA32              res_hor;
-   DATA32              res_ver;
-   DATA32              colors;
-   DATA32              colors_important;
+   uint32_t            header_size;
+   uint32_t            width;
+   uint32_t            height;
+   uint16_t            planes;
+   uint16_t            bpp;
+   uint32_t            compression;
+   uint32_t            size;
+   uint32_t            res_hor;
+   uint32_t            res_ver;
+   uint32_t            colors;
+   uint32_t            colors_important;
 } bih_t;
 
 typedef struct {
@@ -83,13 +85,12 @@ typedef struct {
    unsigned short      w;
    unsigned short      h;
 
-   DATA32             *cmap;    /* Colormap (bpp <= 8) */
-   DATA8              *pxls;    /* Pixel data */
-   DATA8              *mask;    /* Bitmask    */
+   uint32_t           *cmap;    /* Colormap (bpp <= 8) */
+   uint8_t            *pxls;    /* Pixel data */
+   uint8_t            *mask;    /* Bitmask    */
 } ie_t;
 
 typedef struct {
-   FILE               *fp;
    idir_t              idir;    /* ICONDIR */
    ie_t               *ie;      /* Icon entries */
 } ico_t;
@@ -109,8 +110,6 @@ ico_delete(ico_t * ico)
           }
         free(ico->ie);
      }
-
-   free(ico);
 }
 
 static void
@@ -133,8 +132,8 @@ ico_read_idir(ico_t * ico, int ino)
    SWAP_LE_32_INPLACE(ie->ide.size);
    SWAP_LE_32_INPLACE(ie->ide.offs);
 
-   D("Entry %2d: Idir: WxHxD = %dx%dx%d, colors = %d\n",
-     ino, ie->w, ie->h, ie->ide.bpp, ie->ide.colors);
+   DL("Entry %2d: Idir: WxHxD = %dx%dx%d, colors = %d\n",
+      ino, ie->w, ie->h, ie->ide.bpp, ie->ide.colors);
 }
 
 static void
@@ -143,6 +142,10 @@ ico_read_icon(ico_t * ico, int ino)
    ie_t               *ie;
    unsigned int        size;
 
+#ifdef WORDS_BIGENDIAN
+   unsigned int        nr;
+#endif
+
    ie = &ico->ie[ino];
 
    mm_seek(ie->ide.offs);
@@ -169,8 +172,8 @@ ico_read_icon(ico_t * ico, int ino)
         goto bail;
      }
 
-   D("Entry %2d: Icon: WxHxD = %dx%dx%d, colors = %d\n",
-     ino, ie->w, ie->h, ie->bih.bpp, ie->bih.colors);
+   DL("Entry %2d: Icon: WxHxD = %dx%dx%d, colors = %d\n",
+      ino, ie->w, ie->h, ie->bih.bpp, ie->bih.colors);
 
    if (ie->bih.width != ie->w || ie->bih.height != 2 * ie->h)
      {
@@ -187,10 +190,10 @@ ico_read_icon(ico_t * ico, int ino)
      case 1:
      case 4:
      case 8:
-        D("Allocating a %d slot colormap\n", ie->bih.colors);
-        if (UINT_MAX / sizeof(DATA32) < ie->bih.colors)
+        DL("Allocating a %d slot colormap\n", ie->bih.colors);
+        if (UINT_MAX / sizeof(uint32_t) < ie->bih.colors)
            goto bail;
-        size = ie->bih.colors * sizeof(DATA32);
+        size = ie->bih.colors * sizeof(uint32_t);
         ie->cmap = malloc(size);
         if (ie->cmap == NULL)
            goto bail;
@@ -215,7 +218,7 @@ ico_read_icon(ico_t * ico, int ino)
       goto bail;
    if (mm_read(ie->pxls, size))
       goto bail;
-   D("Pixel data size: %u\n", size);
+   DL("Pixel data size: %u\n", size);
 
    size = ((ie->w + 31) / 32 * 4) * ie->h;
    ie->mask = malloc(size);
@@ -223,7 +226,7 @@ ico_read_icon(ico_t * ico, int ino)
       goto bail;
    if (mm_read(ie->mask, size))
       goto bail;
-   D("Mask  data size: %u\n", size);
+   DL("Mask  data size: %u\n", size);
 
    return;
 
@@ -231,50 +234,8 @@ ico_read_icon(ico_t * ico, int ino)
    ie->w = ie->h = 0;           /* Mark invalid */
 }
 
-static ico_t       *
-ico_read(ImlibImage * im, void *data, unsigned int size)
-{
-   ico_t              *ico;
-   unsigned int        i;
-
-   ico = calloc(1, sizeof(ico_t));
-   if (!ico)
-      return NULL;
-
-   ico->fp = im->fp;
-
-   if (mm_read(&ico->idir, sizeof(ico->idir)))
-      goto bail;
-
-   SWAP_LE_16_INPLACE(ico->idir.rsvd);
-   SWAP_LE_16_INPLACE(ico->idir.type);
-   SWAP_LE_16_INPLACE(ico->idir.icons);
-
-   if (ico->idir.rsvd != 0 ||
-       (ico->idir.type != 1 && ico->idir.type != 2) || ico->idir.icons <= 0)
-      goto bail;
-
-   ico->ie = calloc(ico->idir.icons, sizeof(ie_t));
-   if (!ico->ie)
-      goto bail;
-
-   D("Loading '%s' Nicons = %d\n", im->real_file, ico->idir.icons);
-
-   for (i = 0; i < ico->idir.icons; i++)
-     {
-        ico_read_idir(ico, i);
-        ico_read_icon(ico, i);
-     }
-
-   return ico;
-
- bail:
-   ico_delete(ico);
-   return NULL;
-}
-
 static int
-ico_data_get_bit(DATA8 * data, int w, int x, int y)
+ico_data_get_bit(uint8_t * data, int w, int x, int y)
 {
    int                 w32, res;
 
@@ -287,7 +248,7 @@ ico_data_get_bit(DATA8 * data, int w, int x, int y)
 }
 
 static int
-ico_data_get_nibble(DATA8 * data, int w, int x, int y)
+ico_data_get_nibble(uint8_t * data, int w, int x, int y)
 {
    int                 w32, res;
 
@@ -300,56 +261,99 @@ ico_data_get_nibble(DATA8 * data, int w, int x, int y)
 }
 
 static int
-ico_load(ico_t * ico, ImlibImage * im, int load_data)
+_load(ImlibImage * im, int load_data)
 {
-   int                 ic, x, y, w, h, d;
-   DATA32             *cmap;
-   DATA8              *pxls, *mask, *psrc;
+   int                 rc;
+   ico_t               ico;
+   unsigned int        i;
+   int                 ic, x, y, w, h, d, frame;
+   uint32_t           *cmap;
+   uint8_t            *pxls, *mask, *psrc;
    ie_t               *ie;
-   DATA32             *pdst;
-   DATA32              pixel;
+   uint32_t           *pdst;
+   uint32_t            pixel;
+   ImlibImageFrame    *pf;
 
-   /* Find icon with largest size and depth */
-   ic = y = d = 0;
-   for (x = 0; x < ico->idir.icons; x++)
+   rc = LOAD_FAIL;
+
+   mm_init(im->fi->fdata, im->fi->fsize);
+
+   ico.ie = NULL;
+   if (mm_read(&ico.idir, sizeof(ico.idir)))
+      goto quit;
+
+   SWAP_LE_16_INPLACE(ico.idir.rsvd);
+   SWAP_LE_16_INPLACE(ico.idir.type);
+   SWAP_LE_16_INPLACE(ico.idir.icons);
+
+   if (ico.idir.rsvd != 0 ||
+       (ico.idir.type != 1 && ico.idir.type != 2) || ico.idir.icons <= 0)
+      goto quit;
+
+   ico.ie = calloc(ico.idir.icons, sizeof(ie_t));
+   if (!ico.ie)
+      QUIT_WITH_RC(LOAD_OOM);
+
+   D("Loading '%s' Nicons = %d\n", im->fi->name, ico.idir.icons);
+
+   for (i = 0; i < ico.idir.icons; i++)
      {
-        ie = &ico->ie[x];
-        w = ie->w;
-        h = ie->h;
-        if (w * h < y)
-           continue;
-        if (w * h == y && ie->bih.bpp < d)
-           continue;
-        ic = x;
-        y = w * h;
-        d = ie->bih.bpp;
+        ico_read_idir(&ico, i);
+        ico_read_icon(&ico, i);
      }
 
-   /* Enable overriding selected index for debug purposes */
-   if (im->key)
+   rc = LOAD_BADIMAGE;          /* Format accepted */
+
+   frame = im->frame;
+   if (frame > 0)
      {
-        ic = atoi(im->key);
-        if (ic >= ico->idir.icons)
-           return 0;
+        if (frame > 1 && frame > ico.idir.icons)
+           QUIT_WITH_RC(LOAD_BADFRAME);
+
+        pf = __imlib_GetFrame(im);
+        if (!pf)
+           QUIT_WITH_RC(LOAD_OOM);
+
+        pf->frame_count = ico.idir.icons;
      }
 
-   ie = &ico->ie[ic];
+   ic = frame - 1;
+   if (ic < 0)
+     {
+        /* Select default: Find icon with largest size and depth */
+        ic = y = d = 0;
+        for (x = 0; x < ico.idir.icons; x++)
+          {
+             ie = &ico.ie[x];
+             w = ie->w;
+             h = ie->h;
+             if (w * h < y)
+                continue;
+             if (w * h == y && ie->bih.bpp < d)
+                continue;
+             ic = x;
+             y = w * h;
+             d = ie->bih.bpp;
+          }
+     }
+
+   ie = &ico.ie[ic];
 
    w = ie->w;
    h = ie->h;
    if (!IMAGE_DIMENSIONS_OK(w, h))
-      return 0;
+      goto quit;
 
    im->w = w;
    im->h = h;
 
-   SET_FLAG(im->flags, F_HAS_ALPHA);
+   im->has_alpha = 1;
 
    if (!load_data)
-      return 1;
+      QUIT_WITH_RC(LOAD_SUCCESS);
 
    if (!__imlib_AllocateData(im))
-      return 0;
+      QUIT_WITH_RC(LOAD_OOM);
 
    D("Loading icon %d: WxHxD=%dx%dx%d\n", ic, w, h, ie->bih.bpp);
 
@@ -357,62 +361,57 @@ ico_load(ico_t * ico, ImlibImage * im, int load_data)
    pxls = ie->pxls;
    mask = ie->mask;
 
+   pdst = im->data + (h - 1) * w;       /* Start in lower left corner */
+
    switch (ie->bih.bpp)
      {
      case 1:
-        for (y = 0; y < h; y++)
+        for (y = 0; y < h; y++, pdst -= 2 * w)
           {
              for (x = 0; x < w; x++)
                {
-                  pdst = &(im->data[(h - 1 - y) * w + x]);
-
                   pixel = cmap[ico_data_get_bit(pxls, w, x, y)];
                   if (ico_data_get_bit(mask, w, x, y) == 0)
                      pixel |= 0xff000000;
 
-                  *pdst = pixel;
+                  *pdst++ = pixel;
                }
           }
         break;
 
      case 4:
-        for (y = 0; y < h; y++)
+        for (y = 0; y < h; y++, pdst -= 2 * w)
           {
              for (x = 0; x < w; x++)
                {
-                  pdst = &(im->data[(h - 1 - y) * w + x]);
-
                   pixel = cmap[ico_data_get_nibble(pxls, w, x, y)];
                   if (ico_data_get_bit(mask, w, x, y) == 0)
                      pixel |= 0xff000000;
 
-                  *pdst = pixel;
+                  *pdst++ = pixel;
                }
           }
         break;
 
      case 8:
-        for (y = 0; y < h; y++)
+        for (y = 0; y < h; y++, pdst -= 2 * w)
           {
              for (x = 0; x < w; x++)
                {
-                  pdst = &(im->data[(h - 1 - y) * w + x]);
-
                   pixel = cmap[pxls[y * w + x]];
                   if (ico_data_get_bit(mask, w, x, y) == 0)
                      pixel |= 0xff000000;
 
-                  *pdst = pixel;
+                  *pdst++ = pixel;
                }
           }
         break;
 
      default:
-        for (y = 0; y < h; y++)
+        for (y = 0; y < h; y++, pdst -= 2 * w)
           {
              for (x = 0; x < w; x++)
                {
-                  pdst = &(im->data[(h - 1 - y) * w + x]);
                   psrc = &pxls[(y * w + x) * ie->bih.bpp / 8];
 
                   pixel = PIXEL_ARGB(0, psrc[2], psrc[1], psrc[0]);
@@ -421,54 +420,21 @@ ico_load(ico_t * ico, ImlibImage * im, int load_data)
                   else if (ico_data_get_bit(mask, w, x, y) == 0)
                      pixel |= 0xff000000;
 
-                  *pdst = pixel;
+                  *pdst++ = pixel;
                }
           }
         break;
      }
 
-   return 1;
-}
+   rc = LOAD_SUCCESS;
 
-int
-load2(ImlibImage * im, int load_data)
-{
-   int                 rc;
-   void               *fdata;
-   ico_t              *ico;
-
-   rc = LOAD_FAIL;
-
-   fdata = mmap(NULL, im->fsize, PROT_READ, MAP_SHARED, fileno(im->fp), 0);
-   if (fdata == MAP_FAILED)
-      return rc;
-
-   mm_init(fdata, im->fsize);
-
-   ico = ico_read(im, fdata, im->fsize);
-   if (!ico)
-      goto quit;
-
-   if (ico_load(ico, im, load_data))
-     {
-        if (im->lc)
-           __imlib_LoadProgressRows(im, 0, im->h);
-        rc = LOAD_SUCCESS;
-     }
-
-   ico_delete(ico);
+   if (im->lc)
+      __imlib_LoadProgressRows(im, 0, im->h);
 
  quit:
-   if (rc <= 0)
-      __imlib_FreeData(im);
-   if (fdata != MAP_FAILED)
-      munmap(fdata, im->fsize);
+   ico_delete(&ico);
+
    return rc;
 }
 
-void
-formats(ImlibLoader * l)
-{
-   static const char  *const list_formats[] = { "ico" };
-   __imlib_LoaderSetFormats(l, list_formats, ARRAY_SIZE(list_formats));
-}
+IMLIB_LOADER(_formats, _load, NULL);
