@@ -1,4 +1,5 @@
-#include "loader_common.h"
+#include "config.h"
+#include "Imlib2_Loader.h"
 
 #include <jpeglib.h>
 #include <setjmp.h>
@@ -6,10 +7,12 @@
 
 #define DBG_PFX "LDR-jpg"
 
+static const char  *const _formats[] = { "jpg", "jpeg", "jfif", "jfi" };
+
 typedef struct {
    struct jpeg_error_mgr jem;
    sigjmp_buf          setjmp_buffer;
-   DATA8              *data;
+   uint8_t            *data;
 } ImLib_JPEG_data;
 
 static void
@@ -61,32 +64,31 @@ _jdata_init(ImLib_JPEG_data * jd)
    return jem;
 }
 
-int
-load2(ImlibImage * im, int load_data)
+static int
+_load(ImlibImage * im, int load_data)
 {
    int                 w, h, rc;
    struct jpeg_decompress_struct jds;
    ImLib_JPEG_data     jdata;
-   DATA8              *ptr, *line[16];
-   DATA32             *ptr2;
+   uint8_t            *ptr, *line[16];
+   uint32_t           *ptr2;
    int                 x, y, l, scans, inc;
    ExifInfo            ei = { 0 };
 
+   rc = LOAD_FAIL;
+
    /* set up error handling */
    jds.err = _jdata_init(&jdata);
    if (sigsetjmp(jdata.setjmp_buffer, 1))
-     {
-        rc = LOAD_FAIL;
-        goto quit;
-     }
-
-   rc = LOAD_FAIL;
+      QUIT_WITH_RC(LOAD_FAIL);
 
    jpeg_create_decompress(&jds);
-   jpeg_stdio_src(&jds, im->fp);
+   jpeg_mem_src(&jds, im->fi->fdata, im->fi->fsize);
    jpeg_save_markers(&jds, JPEG_APP0 + 1, 256);
    jpeg_read_header(&jds, TRUE);
 
+   rc = LOAD_BADIMAGE;          /* Format accepted */
+
    /* Get orientation */
    ei.orientation = ORIENT_TOPLEFT;
 
@@ -116,13 +118,8 @@ load2(ImlibImage * im, int load_data)
         im->h = h;
      }
 
-   UNSET_FLAG(im->flags, F_HAS_ALPHA);
-
    if (!load_data)
-     {
-        rc = LOAD_SUCCESS;
-        goto quit;
-     }
+      QUIT_WITH_RC(LOAD_SUCCESS);
 
    /* Load data */
 
@@ -135,12 +132,12 @@ load2(ImlibImage * im, int load_data)
 
    jdata.data = malloc(w * 16 * jds.output_components);
    if (!jdata.data)
-      goto quit;
+      QUIT_WITH_RC(LOAD_OOM);
 
    /* must set the im->data member before callign progress function */
    ptr2 = __imlib_AllocateData(im);
    if (!ptr2)
-      goto quit;
+      QUIT_WITH_RC(LOAD_OOM);
 
    for (y = 0; y < jds.rec_outbuf_height; y++)
       line[y] = jdata.data + (y * w * jds.output_components);
@@ -232,11 +229,9 @@ load2(ImlibImage * im, int load_data)
         if (ei.orientation != ORIENT_TOPLEFT &&
             ei.orientation != ORIENT_TOPRIGHT)
            continue;
+
         if (im->lc && __imlib_LoadProgressRows(im, l, scans))
-          {
-             rc = LOAD_BREAK;
-             goto quit;
-          }
+           QUIT_WITH_RC(LOAD_BREAK);
      }
    if (ei.orientation != ORIENT_TOPLEFT && ei.orientation != ORIENT_TOPRIGHT)
      {
@@ -251,34 +246,32 @@ load2(ImlibImage * im, int load_data)
  quit:
    jpeg_destroy_decompress(&jds);
    free(jdata.data);
-   if (rc <= 0)
-      __imlib_FreeData(im);
 
    return rc;
 }
 
-char
-save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
+static int
+_save(ImlibImage * im)
 {
    int                 rc;
    struct jpeg_compress_struct jcs;
    ImLib_JPEG_data     jdata;
    FILE               *f;
-   DATA8              *buf;
-   DATA32             *ptr;
+   uint8_t            *buf;
+   uint32_t           *ptr;
    JSAMPROW           *jbuf;
    int                 y, quality, compression;
    ImlibImageTag      *tag;
    int                 i, j;
 
    /* allocate a small buffer to convert image data */
-   buf = malloc(im->w * 3 * sizeof(DATA8));
+   buf = malloc(im->w * 3 * sizeof(uint8_t));
    if (!buf)
       return LOAD_FAIL;
 
    rc = LOAD_FAIL;
 
-   f = fopen(im->real_file, "wb");
+   f = fopen(im->fi->name, "wb");
    if (!f)
       goto quit;
 
@@ -325,6 +318,11 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    /* set up jepg compression parameters */
    jpeg_set_defaults(&jcs);
    jpeg_set_quality(&jcs, quality, TRUE);
+
+   /* progressive */
+   if ((tag = __imlib_GetTag(im, "interlacing")) && tag->val)
+      jpeg_simple_progression(&jcs);
+
    jpeg_start_compress(&jcs, TRUE);
    /* get the start pointer */
    ptr = im->data;
@@ -334,7 +332,7 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
         /* convcert scaline from ARGB to RGB packed */
         for (j = 0, i = 0; i < im->w; i++)
           {
-             DATA32              pixel = *ptr++;
+             uint32_t            pixel = *ptr++;
 
              buf[j++] = PIXEL_R(pixel);
              buf[j++] = PIXEL_G(pixel);
@@ -345,10 +343,7 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
         jpeg_write_scanlines(&jcs, jbuf, 1);
 
         if (im->lc && __imlib_LoadProgressRows(im, y, 1))
-          {
-             rc = LOAD_BREAK;
-             goto quit;
-          }
+           QUIT_WITH_RC(LOAD_BREAK);
      }
 
    rc = LOAD_SUCCESS;
@@ -363,9 +358,4 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    return rc;
 }
 
-void
-formats(ImlibLoader * l)
-{
-   static const char  *const list_formats[] = { "jpg", "jpeg", "jfif", "jfi" };
-   __imlib_LoaderSetFormats(l, list_formats, ARRAY_SIZE(list_formats));
-}
+IMLIB_LOADER(_formats, _load, _save);
