@@ -1,18 +1,20 @@
-#include "loader_common.h"
+#include "config.h"
+#include "Imlib2_Loader.h"
 
 #include <ctype.h>
 #include <stdbool.h>
-#include <sys/mman.h>
 
 #define DBG_PFX "LDR-pnm"
 
+static const char  *const _formats[] = { "pnm", "ppm", "pgm", "pbm", "pam" };
+
 static struct {
    const unsigned char *data, *dptr;
    unsigned int        size;
 } mdata;
 
 static void
-mm_init(void *src, unsigned int size)
+mm_init(const void *src, unsigned int size)
 {
    mdata.data = mdata.dptr = src;
    mdata.size = size;
@@ -33,7 +35,7 @@ mm_read(void *dst, unsigned int len)
 static int
 mm_getc(void)
 {
-   int                 ch;
+   unsigned char       ch;
 
    if (mdata.dptr + 1 > mdata.data + mdata.size)
       return -1;                /* Out of data */
@@ -112,26 +114,21 @@ mm_getu(unsigned int *pui)
    return 0;                    /* Ok */
 }
 
-int
-load2(ImlibImage * im, int load_data)
+static int
+_load(ImlibImage * im, int load_data)
 {
    int                 rc;
-   void               *fdata;
    int                 c, p;
    int                 w, h, v, numbers, count;
-   DATA8              *data = NULL;     /* for the binary versions */
-   DATA8              *ptr = NULL;
+   uint8_t            *data = NULL;     /* for the binary versions */
+   uint8_t            *ptr = NULL;
    int                *idata = NULL;    /* for the ASCII versions */
-   DATA32             *ptr2, rval, gval, bval;
+   uint32_t           *ptr2, rval, gval, bval;
    int                 i, j, x, y;
 
    rc = LOAD_FAIL;
 
-   fdata = mmap(NULL, im->fsize, PROT_READ, MAP_SHARED, fileno(im->fp), 0);
-   if (fdata == MAP_FAILED)
-      return LOAD_BADFILE;
-
-   mm_init(fdata, im->fsize);
+   mm_init(im->fi->fdata, im->fi->fsize);
 
    /* read the header info */
 
@@ -189,7 +186,7 @@ load2(ImlibImage * im, int load_data)
    if (!IMAGE_DIMENSIONS_OK(w, h))
       goto quit;
 
-   UPDATE_FLAG(im->flags, F_HAS_ALPHA, p == '8');
+   im->has_alpha = p == '8';
 
    if (!load_data)
       QUIT_WITH_RC(LOAD_SUCCESS);
@@ -273,7 +270,7 @@ load2(ImlibImage * im, int load_data)
           }
         break;
      case '4':                 /* binary 1bit monochrome */
-        data = malloc((w + 7) / 8 * sizeof(DATA8));
+        data = malloc((w + 7) / 8 * sizeof(uint8_t));
         if (!data)
            QUIT_WITH_RC(LOAD_OOM);
 
@@ -303,7 +300,7 @@ load2(ImlibImage * im, int load_data)
           }
         break;
      case '5':                 /* binary 8bit grayscale GGGGGGGG */
-        data = malloc(1 * sizeof(DATA8) * w);
+        data = malloc(1 * sizeof(uint8_t) * w);
         if (!data)
            QUIT_WITH_RC(LOAD_OOM);
 
@@ -342,7 +339,7 @@ load2(ImlibImage * im, int load_data)
           }
         break;
      case '6':                 /* 24bit binary RGBRGBRGB */
-        data = malloc(3 * sizeof(DATA8) * w);
+        data = malloc(3 * sizeof(uint8_t) * w);
         if (!data)
            QUIT_WITH_RC(LOAD_OOM);
 
@@ -381,7 +378,7 @@ load2(ImlibImage * im, int load_data)
           }
         break;
      case '7':                 /* XV's 8bit 332 format */
-        data = malloc(1 * sizeof(DATA8) * w);
+        data = malloc(1 * sizeof(uint8_t) * w);
         if (!data)
            QUIT_WITH_RC(LOAD_OOM);
 
@@ -413,7 +410,7 @@ load2(ImlibImage * im, int load_data)
           }
         break;
      case '8':                 /* 24bit binary RGBARGBARGBA */
-        data = malloc(4 * sizeof(DATA8) * w);
+        data = malloc(4 * sizeof(uint8_t) * w);
         if (!data)
            QUIT_WITH_RC(LOAD_OOM);
 
@@ -462,38 +459,34 @@ load2(ImlibImage * im, int load_data)
  quit:
    free(idata);
    free(data);
-   munmap(fdata, im->fsize);
-
-   if (rc == 0)
-      __imlib_FreeData(im);
 
    return rc;
 }
 
-char
-save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
+static int
+_save(ImlibImage * im)
 {
    int                 rc;
    FILE               *f;
-   DATA8              *buf, *bptr;
-   DATA32             *ptr;
+   uint8_t            *buf, *bptr;
+   uint32_t           *ptr;
    int                 x, y;
 
-   f = fopen(im->real_file, "wb");
+   f = fopen(im->fi->name, "wb");
    if (!f)
       return LOAD_FAIL;
 
    rc = LOAD_FAIL;
 
    /* allocate a small buffer to convert image data */
-   buf = malloc(im->w * 4 * sizeof(DATA8));
+   buf = malloc(im->w * 4 * sizeof(uint8_t));
    if (!buf)
       goto quit;
 
    ptr = im->data;
 
    /* if the image has a useful alpha channel */
-   if (im->flags & F_HAS_ALPHA)
+   if (im->has_alpha)
      {
         fprintf(f, "P8\n" "# PNM File written by Imlib2\n" "%i %i\n" "255\n",
                 im->w, im->h);
@@ -502,7 +495,7 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
              bptr = buf;
              for (x = 0; x < im->w; x++)
                {
-                  DATA32              pixel = *ptr++;
+                  uint32_t            pixel = *ptr++;
 
                   bptr[0] = PIXEL_R(pixel);
                   bptr[1] = PIXEL_G(pixel);
@@ -525,7 +518,7 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
              bptr = buf;
              for (x = 0; x < im->w; x++)
                {
-                  DATA32              pixel = *ptr++;
+                  uint32_t            pixel = *ptr++;
 
                   bptr[0] = PIXEL_R(pixel);
                   bptr[1] = PIXEL_G(pixel);
@@ -553,10 +546,4 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    goto quit;
 }
 
-void
-formats(ImlibLoader * l)
-{
-   static const char  *const list_formats[] =
-      { "pnm", "ppm", "pgm", "pbm", "pam" };
-   __imlib_LoaderSetFormats(l, list_formats, ARRAY_SIZE(list_formats));
-}
+IMLIB_LOADER(_formats, _load, _save);
