@@ -1,12 +1,110 @@
-/* To do: */
-/* o Need code to handle tiff with different orientations */
-
-#include "loader_common.h"
+#include "config.h"
+#include "Imlib2_Loader.h"
 
 #include <setjmp.h>
-#include <stdint.h>
 #include <tiffio.h>
-#include <sys/mman.h>
+
+#define DBG_PFX "LDR-tiff"
+
+static const char  *const _formats[] = { "tiff", "tif" };
+
+#define DD(fmt...)  DC(0x80, fmt)
+
+static struct {
+   const unsigned char *data, *dptr;
+   unsigned int        size;
+} mdata;
+
+static void
+mm_init(const void *src, unsigned int size)
+{
+   mdata.data = mdata.dptr = src;
+   mdata.size = size;
+}
+
+static              tmsize_t
+_tiff_read(thandle_t ctx, void *buf, tmsize_t len)
+{
+   DD("%s: len=%ld\n", __func__, len);
+
+   if (mdata.dptr + len > mdata.data + mdata.size)
+      return 0;                 /* Out of data */
+
+   memcpy(buf, mdata.dptr, len);
+   mdata.dptr += len;
+
+   return len;
+}
+
+static              tmsize_t
+_tiff_write(thandle_t ctx, void *buf, tmsize_t len)
+{
+   DD("%s: len=%ld\n", __func__, len);
+
+   return 0;
+}
+
+static              toff_t
+_tiff_seek(thandle_t ctx, toff_t offs, int whence)
+{
+   const unsigned char *dptr;
+
+   DD("%s: offs=%ld, whence=%d\n", __func__, offs, whence);
+
+   switch (whence)
+     {
+     default:
+        return -1;
+     case SEEK_SET:
+        dptr = mdata.data + offs;
+        break;
+     case SEEK_CUR:
+        dptr = mdata.data += offs;
+        break;
+     case SEEK_END:
+        dptr = mdata.data + mdata.size + offs;
+        break;
+     }
+
+   if (dptr > mdata.data + mdata.size)
+      return -1;                /* Out of data */
+
+   mdata.dptr = dptr;
+
+   return mdata.dptr - mdata.data;
+}
+
+static int
+_tiff_close(thandle_t ctx)
+{
+   DD("%s\n", __func__);
+
+   return 0;
+}
+
+static              toff_t
+_tiff_size(thandle_t ctx)
+{
+   DD("%s: size=%d\n", __func__, mdata.size);
+
+   return mdata.size;
+}
+
+static int
+_tiff_map(thandle_t ctx, void **base, toff_t * size)
+{
+   DD("%s\n", __func__);
+
+   *base = (void *)mdata.data;
+   *size = mdata.size;
+
+   return 1;
+}
+static void
+_tiff_unmap(thandle_t ctx, void *base, toff_t size)
+{
+   DD("%s\n", __func__);
+}
 
 /* This is a wrapper data structure for TIFFRGBAImage, so that data can be */
 /* passed into the callbacks. More elegent, I think, than a bunch of globals */
@@ -27,7 +125,7 @@ raster(TIFFRGBAImage_Extra * img, uint32_t * rast,
    uint32_t            image_width, image_height;
    uint32_t           *pixel, pixel_value;
    uint32_t            i, j, k;
-   DATA32             *buffer_pixel, *buffer = img->image->data;
+   uint32_t           *buffer_pixel, *buffer = img->image->data;
    int                 alpha_premult;
    int                 a, r, g, b;
 
@@ -234,11 +332,10 @@ put_separate_and_raster(TIFFRGBAImage * img, uint32_t * rast,
    raster((TIFFRGBAImage_Extra *) img, rast, x, y, w, h);
 }
 
-int
-load2(ImlibImage * im, int load_data)
+static int
+_load(ImlibImage * im, int load_data)
 {
-   int                 rc, fd;
-   void               *fdata;
+   int                 rc;
    TIFF               *tif = NULL;
    uint16_t            magic_number;
    TIFFRGBAImage_Extra rgba_image;
@@ -246,33 +343,26 @@ load2(ImlibImage * im, int load_data)
    char                txt[1024];
 
    rc = LOAD_FAIL;
-   fd = fileno(im->fp);
    rgba_image.image = NULL;
 
    /* Do initial signature check */
 #define TIFF_BYTES_TO_CHECK sizeof(magic_number)
 
-   if (im->fsize < (int)TIFF_BYTES_TO_CHECK)
+   if (im->fi->fsize < (int)TIFF_BYTES_TO_CHECK)
       return rc;
 
-   fdata = mmap(NULL, TIFF_BYTES_TO_CHECK, PROT_READ, MAP_SHARED, fd, 0);
-   if (fdata == MAP_FAILED)
-      return LOAD_BADFILE;
-
-   magic_number = *(uint16_t *) fdata;
-
-   munmap(fdata, TIFF_BYTES_TO_CHECK);
+   magic_number = *(const uint16_t *)im->fi->fdata;
 
    if (magic_number != TIFF_BIGENDIAN && magic_number != TIFF_LITTLEENDIAN)
       return rc;
 
-   fd = dup(fd);
-   tif = TIFFFdOpen(fd, im->real_file, "r");
+   mm_init(im->fi->fdata, im->fi->fsize);
+
+   tif = TIFFClientOpen(im->fi->name, "r", NULL, _tiff_read, _tiff_write,
+                        _tiff_seek, _tiff_close, _tiff_size,
+                        _tiff_map, _tiff_unmap);
    if (!tif)
-     {
-        close(fd);
-        goto quit;
-     }
+      goto quit;
 
    strcpy(txt, "Cannot be processed by libtiff");
    if (!TIFFRGBAImageOK(tif, txt))
@@ -313,8 +403,7 @@ load2(ImlibImage * im, int load_data)
    if (!IMAGE_DIMENSIONS_OK(im->w, im->h))
       goto quit;
 
-   UPDATE_FLAG(im->flags, F_HAS_ALPHA,
-               rgba_image.rgba.alpha != EXTRASAMPLE_UNSPECIFIED);
+   im->has_alpha = rgba_image.rgba.alpha != EXTRASAMPLE_UNSPECIFIED;
 
    if (!load_data)
       QUIT_WITH_RC(LOAD_SUCCESS);
@@ -351,8 +440,6 @@ load2(ImlibImage * im, int load_data)
  quit:
    if (rast)
       _TIFFfree(rast);
-   if (rc <= 0)
-      __imlib_FreeData(im);
    if (rgba_image.image)
       TIFFRGBAImageEnd((TIFFRGBAImage *) & rgba_image);
    if (tif)
@@ -364,25 +451,22 @@ load2(ImlibImage * im, int load_data)
 /* this seems to work, except the magic number isn't written. I'm guessing */
 /* this is a problem in libtiff */
 
-char
-save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
+static int
+_save(ImlibImage * im)
 {
    int                 rc;
    TIFF               *tif = NULL;
    uint8_t            *buf = NULL;
-   DATA32              pixel, *data = im->data;
+   uint32_t            pixel, *data = im->data;
    double              alpha_factor;
    int                 x, y;
    uint8_t             r, g, b, a = 0;
-   int                 has_alpha = IMAGE_HAS_ALPHA(im);
+   int                 has_alpha = im->has_alpha;
+   int                 compression_type;
    int                 i;
-
-   /* By default uses patent-free use COMPRESSION_DEFLATE,
-    * another lossless compression technique */
    ImlibImageTag      *tag;
-   int                 compression_type = COMPRESSION_DEFLATE;
 
-   tif = TIFFOpen(im->real_file, "w");
+   tif = TIFFOpen(im->fi->name, "w");
    if (!tif)
       return LOAD_FAIL;
 
@@ -403,12 +487,14 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    /* saver modules */
 
    /* compression */
+   compression_type = COMPRESSION_ADOBE_DEFLATE;
    tag = __imlib_GetTag(im, "compression_type");
    if (tag)
      {
-        compression_type = tag->val;
-        switch (compression_type)
+        switch (tag->val)
           {
+          default:
+             break;
           case COMPRESSION_NONE:
           case COMPRESSION_CCITTRLE:
           case COMPRESSION_CCITTFAX3:
@@ -432,11 +518,9 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
           case COMPRESSION_JBIG:
           case COMPRESSION_SGILOG:
           case COMPRESSION_SGILOG24:
+             compression_type = tag->val;
              break;
-          default:
-             compression_type = COMPRESSION_DEFLATE;
           }
-
      }
    TIFFSetField(tif, TIFFTAG_COMPRESSION, compression_type);
 
@@ -503,9 +587,4 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    return rc;
 }
 
-void
-formats(ImlibLoader * l)
-{
-   static const char  *const list_formats[] = { "tiff", "tif" };
-   __imlib_LoaderSetFormats(l, list_formats, ARRAY_SIZE(list_formats));
-}
+IMLIB_LOADER(_formats, _load, _save);
