@@ -1,33 +1,31 @@
-#include "loader_common.h"
+#include "config.h"
+#include "Imlib2_Loader.h"
 
-#include <sys/mman.h>
 #include <webp/decode.h>
 #include <webp/demux.h>
 #include <webp/encode.h>
 
 #define DBG_PFX "LDR-webp"
 
-int
-load2(ImlibImage * im, int load_data)
+static const char  *const _formats[] = { "webp" };
+
+static int
+_load(ImlibImage * im, int load_data)
 {
    int                 rc;
-   void               *fdata;
    WebPData            webp_data;
    WebPDemuxer        *demux;
    WebPIterator        iter;
-   int                 frame;
+   int                 frame, fcount;
+   ImlibImageFrame    *pf;
 
    rc = LOAD_FAIL;
 
-   if (im->fsize < 12)
+   if (im->fi->fsize < 12)
       return rc;
 
-   fdata = mmap(NULL, im->fsize, PROT_READ, MAP_SHARED, fileno(im->fp), 0);
-   if (fdata == MAP_FAILED)
-      return LOAD_BADFILE;
-
-   webp_data.bytes = fdata;
-   webp_data.size = im->fsize;
+   webp_data.bytes = im->fi->fdata;
+   webp_data.size = im->fi->fsize;
 
    /* Init (includes signature check) */
    demux = WebPDemux(&webp_data);
@@ -36,21 +34,30 @@ load2(ImlibImage * im, int load_data)
 
    rc = LOAD_BADIMAGE;          /* Format accepted */
 
-   frame = 1;
-   if (im->frame_num > 0)
+   pf = NULL;
+   frame = im->frame;
+   if (frame > 0)
      {
-        frame = im->frame_num;
-        im->frame_count = WebPDemuxGetI(demux, WEBP_FF_FRAME_COUNT);
-        if (im->frame_count > 1)
-           im->frame_flags |= FF_IMAGE_ANIMATED;
-        im->canvas_w = WebPDemuxGetI(demux, WEBP_FF_CANVAS_WIDTH);
-        im->canvas_h = WebPDemuxGetI(demux, WEBP_FF_CANVAS_HEIGHT);
-
-        D("Canvas WxH=%dx%d frames=%d\n",
-          im->canvas_w, im->canvas_h, im->frame_count);
-
-        if (frame > 1 && frame > im->frame_count)
+        fcount = WebPDemuxGetI(demux, WEBP_FF_FRAME_COUNT);
+        if (frame > 1 && frame > fcount)
            QUIT_WITH_RC(LOAD_BADFRAME);
+
+        pf = __imlib_GetFrame(im);
+        if (!pf)
+           QUIT_WITH_RC(LOAD_OOM);
+        pf->frame_count = fcount;
+        pf->loop_count = WebPDemuxGetI(demux, WEBP_FF_LOOP_COUNT);
+        if (pf->frame_count > 1)
+           pf->frame_flags |= FF_IMAGE_ANIMATED;
+        pf->canvas_w = WebPDemuxGetI(demux, WEBP_FF_CANVAS_WIDTH);
+        pf->canvas_h = WebPDemuxGetI(demux, WEBP_FF_CANVAS_HEIGHT);
+
+        D("Canvas WxH=%dx%d frames=%d repeat=%d\n",
+          pf->canvas_w, pf->canvas_h, pf->frame_count, pf->loop_count);
+     }
+   else
+     {
+        frame = 1;
      }
 
    if (!WebPDemuxGetFrame(demux, frame, &iter))
@@ -60,23 +67,27 @@ load2(ImlibImage * im, int load_data)
 
    im->w = iter.width;
    im->h = iter.height;
-   im->frame_x = iter.x_offset;
-   im->frame_y = iter.y_offset;
-   im->frame_delay = iter.duration;
-   if (iter.dispose_method == WEBP_MUX_DISPOSE_BACKGROUND)
-      im->frame_flags |= FF_FRAME_DISPOSE_CLEAR;
-   if (iter.blend_method == WEBP_MUX_BLEND)
-      im->frame_flags |= FF_FRAME_BLEND;
+   if (pf)
+     {
+        pf->frame_x = iter.x_offset;
+        pf->frame_y = iter.y_offset;
+        pf->frame_delay = iter.duration;
+        if (iter.dispose_method == WEBP_MUX_DISPOSE_BACKGROUND)
+           pf->frame_flags |= FF_FRAME_DISPOSE_CLEAR;
+        if (iter.blend_method == WEBP_MUX_BLEND)
+           pf->frame_flags |= FF_FRAME_BLEND;
 
-   D("Canvas WxH=%dx%d frame=%d/%d X,Y=%d,%d WxH=%dx%d alpha=%d T=%d dm=%d co=%d bl=%d\n",      //
-     im->canvas_w, im->canvas_h, iter.frame_num, im->frame_count,
-     im->frame_x, im->frame_y, im->w, im->h, iter.has_alpha,
-     im->frame_delay, iter.dispose_method, iter.complete, iter.blend_method);
+        D("Canvas WxH=%dx%d frame=%d/%d X,Y=%d,%d WxH=%dx%d alpha=%d T=%d dm=%d co=%d bl=%d\n", //
+          pf->canvas_w, pf->canvas_h, iter.frame_num, pf->frame_count,
+          pf->frame_x, pf->frame_y, im->w, im->h, iter.has_alpha,
+          pf->frame_delay, iter.dispose_method, iter.complete,
+          iter.blend_method);
+     }
 
    if (!IMAGE_DIMENSIONS_OK(im->w, im->h))
       goto quit;
 
-   UPDATE_FLAG(im->flags, F_HAS_ALPHA, iter.has_alpha);
+   im->has_alpha = iter.has_alpha;
 
    if (!load_data)
       QUIT_WITH_RC(LOAD_SUCCESS);
@@ -88,26 +99,23 @@ load2(ImlibImage * im, int load_data)
 
    if (WebPDecodeBGRAInto
        (iter.fragment.bytes, iter.fragment.size, (uint8_t *) im->data,
-        sizeof(DATA32) * im->w * im->h, im->w * 4) == NULL)
+        sizeof(uint32_t) * im->w * im->h, im->w * 4) == NULL)
       goto quit;
 
    if (im->lc)
-      __imlib_LoadProgress(im, im->frame_x, im->frame_y, im->w, im->h);
+      __imlib_LoadProgress(im, 0, 0, im->w, im->h);
 
    rc = LOAD_SUCCESS;
 
  quit:
-   if (rc <= 0)
-      __imlib_FreeData(im);
    if (demux)
       WebPDemuxDelete(demux);
-   munmap(fdata, im->fsize);
 
    return rc;
 }
 
-char
-save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
+static int
+_save(ImlibImage * im)
 {
    FILE               *f;
    int                 rc;
@@ -116,7 +124,7 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    uint8_t            *fdata;
    size_t              encoded_size;
 
-   f = fopen(im->real_file, "wb");
+   f = fopen(im->fi->name, "wb");
    if (!f)
       return LOAD_FAIL;
 
@@ -148,7 +156,7 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    encoded_size = WebPEncodeBGRA((uint8_t *) im->data, im->w, im->h,
                                  im->w * 4, quality, &fdata);
 
-   if (fwrite(fdata, encoded_size, 1, f) != encoded_size)
+   if (fwrite(fdata, encoded_size, 1, f) != 1)
       goto quit;
 
    rc = LOAD_SUCCESS;
@@ -161,9 +169,4 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    return rc;
 }
 
-void
-formats(ImlibLoader * l)
-{
-   static const char  *const list_formats[] = { "webp" };
-   __imlib_LoaderSetFormats(l, list_formats, ARRAY_SIZE(list_formats));
-}
+IMLIB_LOADER(_formats, _load, _save);
