@@ -1,7 +1,7 @@
-#define _GNU_SOURCE             /* memmem() */
-#include "loader_common.h"
+#include "config.h"
+#include "Imlib2_Loader.h"
 
-#include <sys/mman.h>
+static const char  *const _formats[] = { "xpm" };
 
 static struct {
    const char         *data, *dptr;
@@ -9,7 +9,7 @@ static struct {
 } mdata;
 
 static void
-mm_init(void *src, unsigned int size)
+mm_init(const void *src, unsigned int size)
 {
    mdata.data = mdata.dptr = src;
    mdata.size = size;
@@ -18,7 +18,7 @@ mm_init(void *src, unsigned int size)
 static int
 mm_getc(void)
 {
-   int                 ch;
+   unsigned char       ch;
 
    if (mdata.dptr + 1 > mdata.data + mdata.size)
       return -1;                /* Out of data */
@@ -31,7 +31,7 @@ mm_getc(void)
 static FILE        *rgb_txt = NULL;
 
 static void
-xpm_parse_color(char *color, DATA32 * pixel)
+xpm_parse_color(char *color, uint32_t * pixel)
 {
    char                buf[4096];
    int                 r, g, b;
@@ -122,7 +122,7 @@ typedef struct {
    char                assigned;
    unsigned char       transp;
    char                str[6];
-   DATA32              pixel;
+   uint32_t            pixel;
 } cmap_t;
 
 static int
@@ -131,7 +131,7 @@ xpm_cmap_sort(const void *a, const void *b)
    return strcmp(((const cmap_t *)a)->str, ((const cmap_t *)b)->str);
 }
 
-static              DATA32
+static              uint32_t
 xpm_cmap_lookup(const cmap_t * cmap, int nc, int cpp, const char *s)
 {
    int                 i, i1, i2, x;
@@ -152,12 +152,11 @@ xpm_cmap_lookup(const cmap_t * cmap, int nc, int cpp, const char *s)
    return cmap[i1].pixel;
 }
 
-int
-load2(ImlibImage * im, int load_data)
+static int
+_load(ImlibImage * im, int load_data)
 {
    int                 rc;
-   void               *fdata;
-   DATA32             *ptr;
+   uint32_t           *ptr;
    int                 pc, c, i, j, k, w, h, ncolors, cpp;
    int                 comment, transp, quote, context, len, done, backslash;
    char               *line, s[256], tok[256], col[256];
@@ -173,14 +172,13 @@ load2(ImlibImage * im, int load_data)
    line = NULL;
    cmap = NULL;
 
-   fdata = mmap(NULL, im->fsize, PROT_READ, MAP_SHARED, fileno(im->fp), 0);
-   if (fdata == MAP_FAILED)
-      return rc;
-
-   if (!memmem(fdata, im->fsize, " XPM */", 7))
+   if (!memmem(im->fi->fdata,
+               im->fi->fsize <= 256 ? im->fi->fsize : 256, " XPM */", 7))
       goto quit;
 
-   mm_init(fdata, im->fsize);
+   rc = LOAD_BADIMAGE;          /* Format accepted */
+
+   mm_init(im->fi->fdata, im->fi->fsize);
 
    j = 0;
    w = 10;
@@ -196,7 +194,7 @@ load2(ImlibImage * im, int load_data)
    count = 0;
    line = malloc(lsz);
    if (!line)
-      goto quit;
+      QUIT_WITH_RC(LOAD_OOM);
    len = 0;
 
    backslash = 0;
@@ -256,7 +254,7 @@ load2(ImlibImage * im, int load_data)
 
                   cmap = calloc(ncolors, sizeof(cmap_t));
                   if (!cmap)
-                     goto quit;
+                     QUIT_WITH_RC(LOAD_OOM);
 
                   pixels = w * h;
 
@@ -360,17 +358,14 @@ load2(ImlibImage * im, int load_data)
                           qsort(cmap, ncolors, sizeof(cmap_t), xpm_cmap_sort);
                        context++;
 
-                       UPDATE_FLAG(im->flags, F_HAS_ALPHA, transp >= 0);
+                       im->has_alpha = transp >= 0;
 
                        if (!load_data)
-                         {
-                            rc = LOAD_SUCCESS;
-                            goto quit;
-                         }
+                          QUIT_WITH_RC(LOAD_SUCCESS);
 
                        ptr = __imlib_AllocateData(im);
                        if (!ptr)
-                          goto quit;
+                          QUIT_WITH_RC(LOAD_OOM);
                     }
                }
              else
@@ -410,10 +405,8 @@ load2(ImlibImage * im, int load_data)
                   if (im->lc && i > last_row)
                     {
                        if (__imlib_LoadProgressRows(im, last_row, i - last_row))
-                         {
-                            rc = LOAD_BREAK;
-                            goto quit;
-                         }
+                          QUIT_WITH_RC(LOAD_BREAK);
+
                        last_row = i;
                     }
                }
@@ -451,7 +444,7 @@ load2(ImlibImage * im, int load_data)
              lsz += 256;
              nline = realloc(line, lsz);
              if (!nline)
-                goto quit;
+                QUIT_WITH_RC(LOAD_OOM);
              line = nline;
           }
 
@@ -469,23 +462,12 @@ load2(ImlibImage * im, int load_data)
    rc = LOAD_SUCCESS;
 
  quit:
-   if (rc <= 0)
-      __imlib_FreeData(im);
-
    free(cmap);
    free(line);
 
    xpm_parse_done();
 
-   if (fdata != MAP_FAILED)
-      munmap(fdata, im->fsize);
-
    return rc;
 }
 
-void
-formats(ImlibLoader * l)
-{
-   static const char  *const list_formats[] = { "xpm" };
-   __imlib_LoaderSetFormats(l, list_formats, ARRAY_SIZE(list_formats));
-}
+IMLIB_LOADER(_formats, _load, NULL);
